/*
 ~ Copyright 2006-2007 Nicolas De Loof.
 ~
 ~ Licensed under the Apache License, Version 2.0 (the "License");
 ~ you may not use this file except in compliance with the License.
 ~ You may obtain a copy of the License at
 ~
 ~      http://www.apache.org/licenses/LICENSE-2.0
 ~
 ~ Unless required by applicable law or agreed to in writing, software
 ~ distributed under the License is distributed on an "AS IS" BASIS,
 ~ WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 ~ See the License for the specific language governing permissions and
 ~ limitations under the License.
 */
package info.jmonit;

import info.jmonit.config.DomainConfig;
import info.jmonit.visitor.RepositoryVisitor;

/**
 * Defines a jMonit Domain.
 * <p>
 * A domain is used to gather Monitoring instances that monitor the same type of
 * data (same unit, same application layer). "web", "service" or "jdbc" are
 * simple domain samples.
 * <p>
 * Monitors from a domain shares the same configuration that is defined at
 * domain level.
 * <p>
 * Domains can be hierachized using a dot notation to share a common
 * configuration and gather incompatible monitors, for example monitors that
 * don't use the same data unit. "soap" can monitor timing of web service calls,
 * and "soap.bytes" the weight of SOAP messages. When disabling "soap", it's
 * "soap.bytes" sub-domain will also be disabled. NOT YET IMPLEMENTED
 *
 * @author <a href="mailto:ndeloof@sourceforge.net">ndeloof</a>
 */
public interface Domain extends Comparable
{
    /**
     * @return true if monitoring is enabled for this domain
     */
    boolean isEnabled();

    /**
     * @return domain name, unique in registry
     */
    String getName();

    /**
     * @param name monitor name
     * @return The monitor uniquely defined by name in this domain
     */
    Monitor getMonitor( String name );

    /**
     * Configure a domain.
     *
     * @param config domain configuration
     */
    void configure( DomainConfig config );

    /**
     * Create a monitor in the domain.
     *
     * @param name monitor name (unique in domain)
     * @return a Monitoring instance
     */
    Monitor createMonitor( String name );

    /**
     * Create a domain as child of this domain
     * @param name subdomain name (without ancestors)
     * @return child domain
     */
    Domain createSubDomain( String name );

    /**
     * @return The domain configuration.
     */
    DomainConfig getConfiguration();

    /**
     * @return parent domain or null.
     */
    Domain getParent();

    /**
     *
     * @param visitor Visitor
     */
    void accept( RepositoryVisitor visitor );

}