/*
 ~ Copyright 2006-2007 Nicolas De Loof.
 ~
 ~ Licensed under the Apache License, Version 2.0 (the "License");
 ~ you may not use this file except in compliance with the License.
 ~ You may obtain a copy of the License at
 ~
 ~      http://www.apache.org/licenses/LICENSE-2.0
 ~
 ~ Unless required by applicable law or agreed to in writing, software
 ~ distributed under the License is distributed on an "AS IS" BASIS,
 ~ WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 ~ See the License for the specific language governing permissions and
 ~ limitations under the License.
 */
package info.jmonit;

import info.jmonit.visitor.MonitorVisitor;

/**
 * A <b>Monitor</b> gather application datas as numeric values. Data unit
 * is not limited to milliseconds so that Monitor can be used to monitor
 * anything usefull (bytes received via a SOAP interface, number of SQL
 * Exceptions...).
 * <p>
 * A Monitor is part of a monitored domain. Monitors in a domain are
 * expected to use the same unit, so that reports can be sortable. jMonit
 * doesn't check for this, as it has no idea of the unit used by the added
 * values.
 *
 * @author <a href="mailto:ndeloof@sourceforge.net">ndeloof</a>
 */
public interface Monitor extends Comparable
{

    /**
     * Add a value to the monitor.
     *
     * @param value numeric valid to be added (positive or negative)
     */
    void add( long value );

    /**
     * Visitor pattern to get into monitor internal state.
     *
     * @param visitor Visitor
     */
    void accept( MonitorVisitor visitor );

    /**
     * @return The domain this monitor is part of.
     */
    Domain getDomain();

    /**
     * @return Name of this monitor, unique in the domain.
     */
    String getName();

    /**
     * Clear monitor datas
     */
    void clear();

    /**
     * Return a Probe for measuring time consumed by a portion of code.
     *
     * @return a Probe
     */
    Probe getProbe();

    /**
     * @param feature Feature name
     * @return true if feature is supported by the monitor
     */
    boolean supports( String feature );
}
