/*
 ~ Copyright 2006-2007 Nicolas De Loof.
 ~
 ~ Licensed under the Apache License, Version 2.0 (the "License");
 ~ you may not use this file except in compliance with the License.
 ~ You may obtain a copy of the License at
 ~
 ~      http://www.apache.org/licenses/LICENSE-2.0
 ~
 ~ Unless required by applicable law or agreed to in writing, software
 ~ distributed under the License is distributed on an "AS IS" BASIS,
 ~ WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 ~ See the License for the specific language governing permissions and
 ~ limitations under the License.
 */
package info.jmonit;

import info.jmonit.config.Bootstrap;

/**
 * <b>Monitoring</b> is a convenience entry point in the jMonit API. It
 * handles a default MonitorRepository and  defines static methods for
 * accessing monitors.
 * <p>
 * The <code>add(..)</code> methods are designed to easily add any numeric
 * value to the specified monitor.
 *
 * <pre>
 * public String callMySaopService( Sring message )
 * {
 *     long bytes = message.getBytes().length;
 *     Monitoring.add( &quot;soap.bytes&quot;, &quot;myService.sent&quot;, bytes );
 *
 *     String reply = service.call( message );
 *
 *     bytes = reply.getBytes().length;
 *     Monitoring.add( &quot;soap.bytes&quot;, &quot;myService.received&quot;, bytes );
 *
 *     return reply;
 * }
 * </pre>
 *
 * In the previous example, the "soap" domain is used to gather datas in bytes.
 * For report consistency, you may ensure all monitors attached to a domain use
 * the same unit, so that the can be compared / ordered in a monitoring report.
 * This is not checked by jMonit (that has no idea of the unit used).
 * <p>
 * A recommended practice is to use a domain hierarchy with dot separator (as in
 * previous code sample). The "root" domain is used for time monitoring, and
 * other monitored datas use a child domain. "soap" domain is used to monitor
 * time spend sending request and wating for web service response, "soap.bytes"
 * gather message weight in bytes, "soap.fault" can be used to monitor SOAP
 * failures, and so on...
 * <p>
 * The <code>start(..)</code> methods can be used to quickly create a Probe
 * and monitor time consumed in a block of code. Never forget to stop the
 * returned Probe ! A try / finally block is the safer way with fiew impact on
 * performances :
 *
 * <pre>
 * public void myBusinessMethod()
 * {
 *     Probe probe = Monitoring.start( &quot;service&quot;, &quot;myBusinessMethod&quot; );
 *     try
 *     {
 *         // Some bunsiness code you want to monitor here...
 *     }
 *     finally
 *     {
 *         probe.stop();
 *     }
 * }
 * </pre>
 *
 * <p>
 * The <code>getMonitor(..)</code> methods are designed to get a Monitoring from
 * the Repository, with no action done on the returned monitor itself. This can
 * be used when user want to access the monitor datas from application code (to
 * build a custom report for example) or to reduce the monitoring cost :
 * <p>
 * Getting the monitor from the Repository requires some synchronization. This
 * doesn't have a high cost for application performances, but getting the
 * monitor every time a new probe is required (or data must be added to monitor)
 * can be avoided by getting the monitor in the class initializer. As Monitoring
 * instance is thread-safe and reusable, it can be stored in a class attribute
 * and used from monitored code. The following code demonstrates a web service
 * call that uses jMonit to gather sent/received bytes.
 *
 * <pre>
 * private final static Monitoring sentMonitor
 *   = Monitoring.getMonitor( &quot;soap.bytes&quot;, &quot;myService.sent&quot; );
 *
 * private final static Monitoring receivedMonitor
 *   = Monitoring.getMonitor( &quot;soap.bytes&quot;, &quot;myService.reveived&quot; );
 *
 * public String callMySaopService( Sring message )
 * {
 *     sentMonitor.add( message.getBytes().length );
 *
 *     String reply = service.call( message );
 *
 *     receivedMonitor.add( reply.getBytes().length );
 *
 *     return reply;
 * }
 * </pre>
 *
 * Please note such optimization is not required by jMonit and will not boost
 * your application performances until you use monitors for every lines of code !
 * <p>
 *
 * @author <a href="mailto:ndeloof@sourceforge.net">ndeloof</a>
 */
public final class Monitoring
{
    /** Default jMonit domain */
    public static final String DEFAULT_DOMAIN = "java";

    /** Repository containing all the monitoring monitors */
    private static Repository repository = Bootstrap.createRepository();

    /**
     * Replace the repository by a custom one.
     * 
     * @param repository the repository to set
     */
    public static void setRepository( Repository repository )
    {
        Monitoring.repository = repository;
    }

    /**
     * @return the repository
     */
    public static Repository getRepository()
    {
        return repository;
    }

    /**
     * Constructor
     */
    private Monitoring()
    {
        super();
    }

    /**
     * Start a new timing Monitoring for the monitor 'name' of the default domain.
     *
     * @param name monitor of moniotring
     * @return a Monitoring instance
     */
    public static Probe start( String name )
    {
        return start( DEFAULT_DOMAIN, name );
    }

    /**
     * Start a new timing Monitoring for the monitor 'domain:name'.
     *
     * @param domain monitor of moniotring
     * @param name monitor of moniotring
     * @return a Monitoring instance
     */
    public static Probe start( String domain, String name )
    {
        Monitor monitor = getMonitor( domain, name );
        Probe probe = monitor.getProbe();
        probe.start();
        return probe;
    }


    /**
     * @return the monitor 'name' of the default domain.
     * @param name The monitor name
     */
    public static Monitor getMonitor( String name )
    {
        return getMonitor( DEFAULT_DOMAIN, name );
    }
    
    /**
     * Get an existing monitor or creates it.
     *
     * @return the monitor 'domain:name'.
     * @param domain The domain name
     * @param name The monitor name
     */
    public static Monitor getMonitor( String domain, String name )
    {
        return (Monitor) getRepository().getMonitor( domain, name );
    }

    /**
     * Add data to the monitor 'name' of the default domain.
     * <p>
     * User should ensure data unit is the same for all monitors used in the
     * domain, so that domain reports display comparable datas.
     * @param name The monitor name
     * @param value The value to add (positive or negative)
     */
    public static void add( String name, long value )
    {
        add( DEFAULT_DOMAIN, name, value );
    }

    /**
     * Add data to the monitor 'domain:name'.
     * <p>
     * User should ensure data unit is the same for all monitors used in the
     * domain, so that domain reports display comparable datas.
     *
     * @param domain The domain name
     * @param name The monitor name
     * @param value The value to add (positive or negative)
     */
    public static void add( String domain, String name, long value )
    {
        getMonitor( domain, name ).add( value );
    }
}
