/*
 ~ Copyright 2006-2007 Nicolas De Loof.
 ~
 ~ Licensed under the Apache License, Version 2.0 (the "License");
 ~ you may not use this file except in compliance with the License.
 ~ You may obtain a copy of the License at
 ~
 ~      http://www.apache.org/licenses/LICENSE-2.0
 ~
 ~ Unless required by applicable law or agreed to in writing, software
 ~ distributed under the License is distributed on an "AS IS" BASIS,
 ~ WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 ~ See the License for the specific language governing permissions and
 ~ limitations under the License.
 */
package info.jmonit;

/**
 * Definition of a Probe for measuring execution time of a process.
 * <p>
 * All Probe method are safe to be used in bad order : you can call
 * <code>stop()</code> or <code>cancel()</code> any time you want without
 * creating duplicated entries in monitoring. You can also pause or resume a
 * non-started monitor without creating phantom datas. Such unexpected calls
 * will simply be ignored.
 *
 * @author <a href="mailto:ndeloof@sourceforge.net">ndeloof</a>
 */
public interface Probe
{
    /**
     * Start monitoring the process.
     *
     * @return start time
     */
    long start();

    /**
     * @return The time this probe was started.
     */
    long startedAt();

    /**
     * Stop monitoring the process. The monitor cannot be re-used after stopped
     * has been called.
     *
     * @return stop time
     */
    long stop();

    /**
     * Temporary stop the monitoring. Elapsed time calculation will not include
     * time spent in paused mode.
     */
    void pause();

    /**
     * Resume the monitoring after a pause.
     */
    void resume();

    /**
     * Cancel monitoring. Elapsed time will not be computed and will not be
     * merged in the monitor monitor.
     */
    void cancel();

    /**
     * @return Elapsed time for the monitored process.
     */
    long getElapsedTime();

}