/*
 ~ Copyright 2006-2007 Nicolas De Loof.
 ~
 ~ Licensed under the Apache License, Version 2.0 (the "License");
 ~ you may not use this file except in compliance with the License.
 ~ You may obtain a copy of the License at
 ~
 ~      http://www.apache.org/licenses/LICENSE-2.0
 ~
 ~ Unless required by applicable law or agreed to in writing, software
 ~ distributed under the License is distributed on an "AS IS" BASIS,
 ~ WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 ~ See the License for the specific language governing permissions and
 ~ limitations under the License.
 */
package info.jmonit;

import info.jmonit.visitor.RepositoryVisitor;

/**
 * A <b>Repository</b> contains all the monitors used by the application and
 * provides a centralized way to acces them. A monitor is defined in the
 * repository by the pair domain + name.
 * <p>
 *
 * @author <a href="mailto:ndeloof@sourceforge.net">ndeloof</a>
 */
public interface Repository
{
    /**
     * @param domain jMonit domain name
     * @param name monitor name
     * @return The monitor uniquely defines by domain + monitor
     */
    Monitor getMonitor( String domain, String name );

    /**
     *
     * @param visitor Visitor
     */
    void accept( RepositoryVisitor visitor );

    /**
     *
     * @param name domain name
     * @return The specified domain
     */
    Domain getDomain( String name );


    /**
     * Register a new monitor instance
     *
     * @param monitor
     *            new instance
     */
    void register( Monitor monitor );

    /**
     * Register a new domain instance
     * @param domain new instance
     */
    void register( Domain domain );


    /**
     * Key for identity of a monitor in the repository.
     *
     * @author ndeloof
     */
    public static final class Key
    {
        /** domain monitor */
        private final String domain;

        /** monitor name */
        private final String name;

        /**
         * Constructor
         *
         * @param domain domain name
         * @param name monitor name
         */
        public Key( String domain, String name )
        {
            super();
            this.domain = domain;
            this.name = name;
        }

        /** Prime number for building hashcode */
        private static final int PRIME = 31;

        /**
         * {@inheritDoc}
         *
         * @see java.lang.Object#hashCode()
         */
        public int hashCode()
        {
            int result = ( domain == null ? 0 : domain.hashCode() );
            result = PRIME * result + ( name == null ? 0 : name.hashCode() );
            return result;
        }

        /**
         * {@inheritDoc}
         *
         * @see java.lang.Object#equals(java.lang.Object)
         */
        public boolean equals( Object obj )
        {
            Key other = (Key) obj;
            return domain.equals( other.domain ) && name.equals( other.name );
        }

        /**
         * @return the domain name
         */
        public String getDomain()
        {
            return domain;
        }

        /**
         * @return the monitor name
         */
        public String getName()
        {
            return name;
        }

        /**
         * {@inheritDoc}
         *
         * @see java.lang.Object#toString()
         */
        public String toString()
        {
            return new StringBuffer( domain ).append( ':' ).append( name ).toString();
        }
    }
}