/*
 ~ Copyright 2006-2007 Nicolas De Loof.
 ~
 ~ Licensed under the Apache License, Version 2.0 (the "License");
 ~ you may not use this file except in compliance with the License.
 ~ You may obtain a copy of the License at
 ~
 ~      http://www.apache.org/licenses/LICENSE-2.0
 ~
 ~ Unless required by applicable law or agreed to in writing, software
 ~ distributed under the License is distributed on an "AS IS" BASIS,
 ~ WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 ~ See the License for the specific language governing permissions and
 ~ limitations under the License.
 */
package info.jmonit.config;

import java.net.MalformedURLException;
import java.net.URL;
import java.text.ParseException;

import info.jmonit.Repository;
import info.jmonit.config.parsers.ConfigurationParser;
import info.jmonit.logger.Logger;
import info.jmonit.logger.LoggerFactory;
import info.jmonit.monitor.DefaultRepository;

/**
 * Bootstrap process to build the default Monitor defaultRepository.
 * <p>
 * The bootstrap will first locate the configuration file:
 * <ul>
 * <li>As an URL defined by <code>jmonit.configuration</code> system property
 * </li>
 * <li>As a Classpath resource named <code>jmonit.xml</code></li>
 * <li>Using the default configuration</li>
 * </ul>
 * The file is parsed and used to create a RepositoryConfig instance. This
 * object is passed to the DefaultRepository instance that configure itself.
 * <p>
 * If anything is wrong during this bootstrap process (IO error, invalid XML
 * syntax...) jMonit will log ERRORS, fall back to default configuration, but
 * never halt the application startup or return a broken Repository.
 *
 * @author <a href="mailto:ndeloof@sourceforge.net">ndeloof</a>
 */
public final class Bootstrap
{
    /** System property for configuration URL */
    private static final String CONFIGURATION = "jmonit.configuration";

    /** Logger */
    private static Logger logger = LoggerFactory.getLogger( Bootstrap.class );

    /** Default defaultRepository */
    private static DefaultRepository defaultRepository = new DefaultRepository();

    /**
     * Build and configure the defaultRepository
     *
     * @return defaultRepository
     */
    public static synchronized Repository createRepository()
    {
        if ( defaultRepository != null )
        {
            return defaultRepository;
        }
        
        String location = System.getProperty( CONFIGURATION );
        if ( location != null )
        {
            try
            {
                URL url = new URL( location );
                ConfigurationParser parser = null;
                RepositoryConfig config = parser.parse( url );
                defaultRepository.configure( config );
            }
            catch ( MalformedURLException e )
            {
                logger.warn( "Invalid value for jmonit.configuration. An URL is required", e );
            }
            catch ( ParseException e )
            {
                logger.warn( "Invalid jmonit configuration file", e );
            }
        }
        return defaultRepository;
    }

}
