/*
 ~ Copyright 2006-2007 Nicolas De Loof.
 ~
 ~ Licensed under the Apache License, Version 2.0 (the "License");
 ~ you may not use this file except in compliance with the License.
 ~ You may obtain a copy of the License at
 ~
 ~      http://www.apache.org/licenses/LICENSE-2.0
 ~
 ~ Unless required by applicable law or agreed to in writing, software
 ~ distributed under the License is distributed on an "AS IS" BASIS,
 ~ WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 ~ See the License for the specific language governing permissions and
 ~ limitations under the License.
 */
package info.jmonit.config;

import java.util.Properties;

/**
 * Configuration information for a Domain
 *
 * @author <a href="mailto:ndeloof@sourceforge.net">ndeloof</a>
 */
public class DomainConfig
{
    /** Domain name */
    private String name;

    /** Domain attributes */
    private Properties attributes;

    /** Configuration of the parent domain */
    private DomainConfig parentConfig;
    
    /** Cache the computed "enabled" property for performances */
    private Boolean enabled;

    /**
     * Constructor
     *
     * @param name Domain name
     */
    public DomainConfig( String name )
    {
        super();
        this.name = name;
        this.attributes = new Properties();
    }

    /**
     * Constructor
     *
     * @param name Domain name
     * @param attributes configuration attributes
     */
    public DomainConfig( String name, Properties attributes )
    {
        this( name );
        this.attributes = attributes;
        if ( attributes.containsKey( "enabled" ) )
        {
            this.enabled = Boolean.valueOf( attributes.getProperty( "enabled" ) );
        }
    }

    /**
     * @return The name.
     */
    public String getName()
    {
        return name;
    }

    /**
     * @return The parent domain name.
     */
    public String getParentName()
    {
        int i = name.indexOf( '.' );
        if ( i > 0 )
        {
            return name.substring( 0, i );
        }
        // Top level domain
        return null;
    }

    /**
     * @return The enable flag.
     */
    public Boolean isEnabled()
    {
        return enabled;
    }

    /**
     * @return the attributes
     */
    public Properties getAttributes()
    {
        return attributes;
    }

    /**
     * @return the parentConfig
     */
    public DomainConfig getParentConfig()
    {
        return parentConfig;
    }

    /**
     * @param parentConfig the parentConfig to set
     */
    public void setParentConfig( DomainConfig parentConfig )
    {
        this.parentConfig = parentConfig;
    }

    /**
     * {@inheritDoc}
     * @see java.util.Properties#getProperty(java.lang.String)
     */
    public String getProperty( String key )
    {
        return attributes.getProperty( key );
    }

    /**
     * {@inheritDoc}
     * @see java.util.Properties#setProperty(java.lang.String, java.lang.String)
     */
    public Object setProperty( String key, String value )
    {
        if ( key.equals( "enabled" ) )
        {
            this.enabled = Boolean.valueOf( attributes.getProperty( "enabled" ) );
        }
        return attributes.setProperty( key, value );
    }
}
