/*
 ~ Copyright 2006-2007 Nicolas De Loof.
 ~
 ~ Licensed under the Apache License, Version 2.0 (the "License");
 ~ you may not use this file except in compliance with the License.
 ~ You may obtain a copy of the License at
 ~
 ~      http://www.apache.org/licenses/LICENSE-2.0
 ~
 ~ Unless required by applicable law or agreed to in writing, software
 ~ distributed under the License is distributed on an "AS IS" BASIS,
 ~ WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 ~ See the License for the specific language governing permissions and
 ~ limitations under the License.
 */
package info.jmonit.config;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

/**
 * Repository configuration datas.
 *
 * @author <a href="mailto:ndeloof@sourceforge.net">ndeloof</a>
 */
public class RepositoryConfig
{
    /** Domains configuration */
    private Map domains;

    /** Extensions configuration */
    private ExtensionConfig[] extensions;

    /**
     * Constructor
     */
    public RepositoryConfig()
    {
        super();
        domains = new HashMap();
    }

    /**
     * Add a DomainConfig to the Repository configuration.
     *
     * @param domainConfig domain configuration
     */
    public void addDomainConfig( DomainConfig domainConfig )
    {
        domains.put( domainConfig.getName(), domainConfig );
    }

    /**
     * Create the domain hierarchy. All parsed domains are linked according to
     * the dotted-name hierachy.
     */
    public void createDomainsHierarchy()
    {
        Collection configuredDomains = new ArrayList( domains.values() );
        for ( Iterator iter = configuredDomains.iterator(); iter.hasNext(); )
        {
            DomainConfig config = (DomainConfig) iter.next();
            createHierarchy( config );
        }
    }

    /**
     * Find an existing or create a DomainConfig
     *
     * @param name domain name
     * @return DomainConfig for the domain
     */
    public DomainConfig getDomainConfig( String name )
    {
        DomainConfig config = (DomainConfig) domains.get( name );
        if ( config == null )
        {
            // Creates an empty domain configuration
            config = new DomainConfig( name );
            domains.put( name, config );
        }
        return config;
    }

    /**
     * Create the domain hierarchy.
     *
     * @param domainConfig domain configuration
     */
    private void createHierarchy( DomainConfig domainConfig )
    {
        String parent = domainConfig.getParentName();
        if ( parent != null )
        {
            DomainConfig parentConfig = getDomainConfig( parent );
            domainConfig.setParentConfig( parentConfig );
            createHierarchy( parentConfig );
        }
    }

    /**
     * @return the domains as Collection<DomainConfig>
     */
    public Collection getConfiguredDomains()
    {
        return domains.entrySet();
    }

    /**
     * @return the extensions
     */
    public ExtensionConfig[] getExtensions()
    {
        return extensions;
    }

    /**
     * @param extensions the extensions to set
     */
    public void setExtensions( ExtensionConfig[] extensions )
    {
        this.extensions = extensions;
    }

}
