/*
 ~ Copyright 2006-2007 Nicolas De Loof.
 ~
 ~ Licensed under the Apache License, Version 2.0 (the "License");
 ~ you may not use this file except in compliance with the License.
 ~ You may obtain a copy of the License at
 ~
 ~      http://www.apache.org/licenses/LICENSE-2.0
 ~
 ~ Unless required by applicable law or agreed to in writing, software
 ~ distributed under the License is distributed on an "AS IS" BASIS,
 ~ WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 ~ See the License for the specific language governing permissions and
 ~ limitations under the License.
 */
package info.jmonit.config.parsers;

import info.jmonit.config.DomainConfig;
import info.jmonit.config.ExtensionConfig;
import info.jmonit.config.RepositoryConfig;
import info.jmonit.logger.Logger;
import info.jmonit.logger.LoggerFactory;

import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.Reader;
import java.net.URL;
import java.text.ParseException;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Properties;
import java.util.Stack;

/**
 * Parse a jMonit XML configuration file.
 * <p>
 * A Quick & Dirty parser is used to avoir dependency to XML apis. This will not
 * allow any validation or advanced XML feature.
 *
 * @author <a href="mailto:ndeloof@sourceforge.net">ndeloof</a>
 */
public class QDConfigurationParser
    implements DocHandler, ConfigurationParser
{
    /** Logger */
    private static Logger logger = LoggerFactory.getLogger( QDConfigurationParser.class );

    /** Domains configuration */
    private List domains;

    /** Hierarchy of domains */
    private Stack domainsHierarchy = new Stack();

    /** Extensions configuration */
    private List extensions;

    /**
     * {@inheritDoc}
     *
     * @see info.jmonit.config.parsers.ConfigurationParser#parse(java.net.URL)
     */
    public RepositoryConfig parse( URL config )
        throws ParseException
    {
        try
        {
            InputStream input = config.openStream();
            Reader reader = new InputStreamReader( input );
            new QDParser().parse( this, reader );
            input.close();
        }
        catch ( IOException e )
        {
            logger.error( "I/O exception accessing " + config, e );
            throw new ParseException( "I/O exception accessing " + config, 0 );
        }
        catch ( Exception e )
        {
            logger.error( "Failed to parse " + config, e );
            throw new ParseException( "Failed to parse " + config, 0 );
        }

        RepositoryConfig repository = new RepositoryConfig();
        for ( Iterator iter = domains.iterator(); iter.hasNext(); )
        {
            repository.addDomainConfig( (DomainConfig) iter.next() );
        }
        repository.setExtensions( (ExtensionConfig[]) extensions.toArray( new ExtensionConfig[extensions.size()] ) );

        repository.createDomainsHierarchy();
        return repository;
    }

    /**
     * {@inheritDoc}
     *
     * @see info.jmonit.config.parsers.DocHandler#startDocument()
     */
    public void startDocument()
        throws Exception
    {
        domains = new ArrayList();
        extensions = new ArrayList();
    }

    /**
     * {@inheritDoc}
     *
     * @see info.jmonit.config.parsers.DocHandler#startElement(java.lang.String,
     * java.util.Properties)
     */
    public void startElement( String tag, Properties attributes )
        throws Exception
    {
        if ( "domain".equals( tag ) )
        {
            String name = (String) attributes.remove( "name" );
            if ( !domainsHierarchy.isEmpty() )
            {
                DomainConfig parentDomain = (DomainConfig) domainsHierarchy.peek();
                name = parentDomain.getName() + "." + name;
            }
            DomainConfig domain = new DomainConfig( name, attributes );
            domainsHierarchy.push( domain );
            domains.add( domain );
        }
        if ( "extension".equals( tag ) )
        {
            extensions.add( new ExtensionConfig( attributes ) );
        }
    }

    /**
     * {@inheritDoc}
     *
     * @see info.jmonit.config.parsers.DocHandler#endElement(java.lang.String)
     */
    public void endElement( String tag )
        throws Exception
    {
        if ( "domain".equals( tag ) )
        {
            domainsHierarchy.pop();
        }
    }

    /**
     * {@inheritDoc}
     *
     * @see info.jmonit.config.parsers.DocHandler#text(java.lang.String)
     */
    public void text( String str )
        throws Exception
    {
        // Nop
    }

    /**
     * {@inheritDoc}
     *
     * @see info.jmonit.config.parsers.DocHandler#endDocument()
     */
    public void endDocument()
        throws Exception
    {
        // Nop
    }

    /**
     * @return The domain configuration
     */
    public List getDomainsConfig()
    {
        return domains;
    }
}
