/*
 ~ Copyright 2006-2007 Nicolas De Loof.
 ~
 ~ Licensed under the Apache License, Version 2.0 (the "License");
 ~ you may not use this file except in compliance with the License.
 ~ You may obtain a copy of the License at
 ~
 ~      http://www.apache.org/licenses/LICENSE-2.0
 ~
 ~ Unless required by applicable law or agreed to in writing, software
 ~ distributed under the License is distributed on an "AS IS" BASIS,
 ~ WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 ~ See the License for the specific language governing permissions and
 ~ limitations under the License.
 */
package info.jmonit.config.parsers;

import java.io.Reader;
import java.util.Properties;
import java.util.Stack;

/**
 * Quick and Dirty xml parser. This parser is, like the SAX parser, an event
 * based parser, but with much less functionality.
 * <p>
 * This parser is used when no XML-api support is available at runtime.
 *
 * @see http://www.javaworld.com/javatips/jw-javatip128_p.html
 */
public class QDParser
{
    /**
     * Pop element from stack
     *
     * @param st stack
     * @return element type
     */
    private static int popMode( Stack st )
    {
        if ( !st.empty() )
        {
            return ( (Integer) st.pop() ).intValue();
        }
        return PRE;
    }

    /** Text node */
    private static final int TEXT = 1;

    /** Entity */
    private static final int ENTITY = 2;

    /** Opening tag */
    private static final int OPEN_TAG = 3;

    /** Closing tag */
    private static final int CLOSE_TAG = 4;

    /** Stat tag */
    private static final int START_TAG = 5;

    /** Attribute name */
    private static final int ATTRIBUTE_LVALUE = 6;

    /** Attribute declaration equal */
    private static final int ATTRIBUTE_EQUAL = 9;

    /** Attribute value */
    private static final int ATTRIBUTE_RVALUE = 10;

    /** Quote */
    private static final int QUOTE = 7;

    /** inside an tag */
    private static final int IN_TAG = 8;

    /** single tag */
    private static final int SINGLE_TAG = 12;

    /** comment */
    private static final int COMMENT = 13;

    /** document end */
    private static final int DONE = 11;

    /** Doctype declaration */
    private static final int DOCTYPE = 14;

    /** before document root element */
    private static final int PRE = 15;

    /** CDATA block */
    private static final int CDATA = 16;

    /** Parsed elements stack */
    private Stack st = new Stack();

    /** Depth in nested XML elements */
    private int depth = 0;

    /** Parsing mode */
    private int mode = PRE;

    /** Quote char (" or ') */
    private int quotec = '"';

    /** Position in document */
    private int line = 1;

    /** Position in document */
    private int col = 0;

    /** end of line ? */
    private boolean eol = false;

    /** Current element beeing parsed */
    private StringBuffer sb = new StringBuffer();

    /** End tag name */
    private StringBuffer etag = new StringBuffer();

    /** tag name */
    private String tagName = null;

    /** left value of an XML attribute */
    private String lvalue = null;

    /** right value of an XML attribute */
    private String rvalue = null;

    /** Current element attributes */
    private Properties attrs = null;

    /**
     * Parse a document
     *
     * @param doc handler for document
     * @param r input reader
     * @throws Exception parse error
     */
    public void parse( DocHandler doc, Reader r )
        throws Exception
    {
        doc.startDocument();
        int c = 0;
        while ( ( c = r.read() ) != -1 )
        {
            // We need to map \r, \r\n, and \n to \n
            // See XML spec section 2.11
            if ( c == '\n' && eol )
            {
                eol = false;
                continue;
            }
            else if ( eol )
            {
                eol = false;
            }
            else if ( c == '\n' )
            {
                line++;
                col = 0;
            }
            else if ( c == '\r' )
            {
                eol = true;
                c = '\n';
                line++;
                col = 0;
            }
            else
            {
                col++;
            }
            if ( mode == DONE )
            {
                doc.endDocument();
                return;

            }
            else if ( mode == TEXT )
            {
                // We are between tags collecting text.
                textNode( doc, c );
            }
            else if ( mode == CLOSE_TAG )
            {
                // we are processing a closing tag: e.g. </foo>
                closeTag( doc, c );
            }
            else if ( mode == CDATA )
            {
                // we are processing CDATA
                cdataSection( doc, c );
            }
            else if ( mode == COMMENT )
            {
                // we are processing a comment. We are inside
                // the <!-- .... --> looking for the -->.
                comment( c );
            }
            else if ( mode == PRE )
            {
                // We are outside the root tag element
                beforeRoot( c );
            }
            else if ( mode == DOCTYPE )
            {
                // We are inside one of these <? ... ?>
                // or one of these <!DOCTYPE ... >
                doctype( c );
            }
            else if ( mode == START_TAG )
            {
                // we have just seen a < and
                // are wondering what we are looking at
                // <foo>, </foo>, <!-- ... --->, etc.
                startTag( c );
            }
            else if ( mode == ENTITY )
            {
                // we are processing an entity, e.g. &lt;, &#187;, etc.
                entityRef( c );
            }
            else if ( mode == SINGLE_TAG )
            {
                // we have just seen something like this:
                // <foo a="b"/
                // and are looking for the final >.
                if ( tagName == null )
                {
                    tagName = sb.toString();
                }
                if ( c != '>' )
                {
                    exc( "Expected > for tag: <" + tagName + "/>" );
                }
                doc.startElement( tagName, attrs );
                doc.endElement( tagName );
                if ( depth == 0 )
                {
                    doc.endDocument();
                    return;
                }
                sb.setLength( 0 );
                attrs = new Properties();
                tagName = null;
                mode = popMode( st );

            }
            else if ( mode == OPEN_TAG )
            {
                // we are processing something
                // like this <foo ... >. It could
                // still be a <!-- ... --> or something.
                openTag( doc, c );
            }
            else if ( mode == QUOTE )
            {
                // We are processing the quoted right-hand side
                // of an element's attribute.
                quotedValue( c );
            }
            else if ( mode == ATTRIBUTE_RVALUE )
            {
                attributeValue( c );
            }
            else if ( mode == ATTRIBUTE_LVALUE )
            {
                attributeName( c );
            }
            else if ( mode == ATTRIBUTE_EQUAL )
            {
                attributeEquals( c );
            }
            else if ( mode == IN_TAG )
            {
                tagContent( doc, c );
            }
        }
        if ( mode == DONE )
        {
            doc.endDocument();
        }
        else
        {
            exc( "missing end tag" );
        }
    }

    /**
     * Content off an XML element
     *
     * @param doc document content handler
     * @param c current char
     * @throws Exception parse error
     */
    private void tagContent( DocHandler doc, int c )
        throws Exception
    {
        if ( c == '>' )
        {
            mode = popMode( st );
            doc.startElement( tagName, attrs );
            depth++;
            tagName = null;
            attrs = new Properties();
        }
        else if ( c == '/' )
        {
            mode = SINGLE_TAG;
        }
        else if ( !Character.isWhitespace( (char) c ) )
        {
            mode = ATTRIBUTE_LVALUE;
            sb.append( (char) c );
        }
    }

    /**
     * "=" between attribute name and quoted value
     *
     * @param c current char
     * @throws Exception parse error
     */
    private void attributeEquals( int c )
        throws Exception
    {
        if ( c == '=' )
        {
            mode = ATTRIBUTE_RVALUE;
        }
        else if ( !Character.isWhitespace( (char) c ) )
        {
            exc( "Error in attribute processing." );
        }
    }

    /**
     * Part of the attribute name.
     *
     * @param c current char
     */
    private void attributeName( int c )
    {
        if ( Character.isWhitespace( (char) c ) )
        {
            lvalue = sb.toString();
            sb.setLength( 0 );
            mode = ATTRIBUTE_EQUAL;
        }
        else if ( c == '=' )
        {
            lvalue = sb.toString();
            sb.setLength( 0 );
            mode = ATTRIBUTE_RVALUE;
        }
        else
        {
            sb.append( (char) c );
        }
    }

    /**
     * Part of the attribute value.
     *
     * @param c current char
     * @throws Exception parse error
     */
    private void attributeValue( int c )
        throws Exception
    {
        if ( c == '"' || c == '\'' )
        {
            quotec = c;
            mode = QUOTE;
        }
        else if ( !Character.isWhitespace( (char) c ) )
        {
            exc( "Error in attribute processing" );
        }
    }

    /**
     * Part of a quoted value.
     *
     * @param c current char
     */
    private void quotedValue( int c )
    {
        if ( c == quotec )
        {
            rvalue = sb.toString();
            sb.setLength( 0 );
            attrs.setProperty( lvalue, rvalue );
            mode = IN_TAG;
            // See section the XML spec, section 3.3.3
            // on normalization processing.
        }
        else if ( " \r\n\u0009".indexOf( c ) >= 0 )
        {
            sb.append( ' ' );
        }
        else if ( c == '&' )
        {
            st.push( new Integer( mode ) );
            mode = ENTITY;
            etag.setLength( 0 );
        }
        else
        {
            sb.append( (char) c );
        }
    }

    /**
     * Inside an open tag (XML element started)
     *
     * @param doc document content handler
     * @param c current char
     * @throws Exception parse error
     */
    private void openTag( DocHandler doc, int c )
        throws Exception
    {
        if ( c == '>' )
        {
            if ( tagName == null )
            {
                tagName = sb.toString();
            }
            sb.setLength( 0 );
            depth++;
            doc.startElement( tagName, attrs );
            tagName = null;
            attrs = new Properties();
            mode = popMode( st );
        }
        else if ( c == '/' )
        {
            mode = SINGLE_TAG;
        }
        else if ( c == '-' && sb.toString().equals( "!-" ) )
        {
            mode = COMMENT;
        }
        else if ( c == '[' && sb.toString().equals( "![CDATA" ) )
        {
            mode = CDATA;
            sb.setLength( 0 );
        }
        else if ( c == 'E' && sb.toString().equals( "!DOCTYP" ) )
        {
            sb.setLength( 0 );
            mode = DOCTYPE;
        }
        else if ( Character.isWhitespace( (char) c ) )
        {
            tagName = sb.toString();
            sb.setLength( 0 );
            mode = IN_TAG;
        }
        else
        {
            sb.append( (char) c );
        }
    }

    /**
     * Reference to an XML Entity.
     *
     * @param c current char
     * @throws Exception parse error
     */
    private void entityRef( int c )
        throws Exception
    {
        if ( c == ';' )
        {
            mode = popMode( st );
            String cent = etag.toString();
            etag.setLength( 0 );
            if ( cent.equals( "lt" ) )
            {
                sb.append( '<' );
            }
            else if ( cent.equals( "gt" ) )
            {
                sb.append( '>' );
            }
            else if ( cent.equals( "amp" ) )
            {
                sb.append( '&' );
            }
            else if ( cent.equals( "quot" ) )
            {
                sb.append( '"' );
            }
            else if ( cent.equals( "apos" ) )
            {
                sb.append( '\'' );
            }
            // Could parse hex entities if we wanted to
            // else if(cent.startsWith("#x"))
            // sb.append((char)Integer.parseInt(cent.substring(2),16));
            else if ( cent.startsWith( "#" ) )
            {
                sb.append( (char) Integer.parseInt( cent.substring( 1 ) ) );
            }
            // Insert custom entity definitions here
            else
            {
                exc( "Unknown entity: &" + cent + ";" );
            }
        }
        else
        {
            etag.append( (char) c );
        }
    }

    /**
     * XML Element start tag.
     *
     * @param c current char
     */
    private void startTag( int c )
    {
        mode = popMode( st );
        if ( c == '/' )
        {
            st.push( new Integer( mode ) );
            mode = CLOSE_TAG;
        }
        else if ( c == '?' )
        {
            mode = DOCTYPE;
        }
        else
        {
            st.push( new Integer( mode ) );
            mode = OPEN_TAG;
            tagName = null;
            attrs = new Properties();
            sb.append( (char) c );
        }
    }

    /**
     * XML Doctype.
     *
     * @param c current char
     */
    private void doctype( int c )
    {
        if ( c == '>' )
        {
            mode = popMode( st );
            if ( mode == TEXT )
            {
                mode = PRE;
            }
        }
    }

    /**
     * Before parsing the XML root element.
     *
     * @param c current char
     */
    private void beforeRoot( int c )
    {
        if ( c == '<' )
        {
            mode = TEXT;
            st.push( new Integer( mode ) );
            mode = START_TAG;
        }
    }

    /**
     * XML comment
     *
     * @param c current char
     */
    private void comment( int c )
    {
        if ( c == '>' && sb.toString().endsWith( "--" ) )
        {
            sb.setLength( 0 );
            mode = popMode( st );
        }
        else
        {
            sb.append( (char) c );
        }
    }

    /**
     * inside a [CDATA] section.
     *
     * @param doc document content handler
     * @param c current char
     * @throws Exception parse error
     */
    private void cdataSection( DocHandler doc, int c )
        throws Exception
    {
        if ( c == '>' && sb.toString().endsWith( "]]" ) )
        {
            sb.setLength( sb.length() - 2 );
            doc.text( sb.toString() );
            sb.setLength( 0 );
            mode = popMode( st );
        }
        else
        {
            sb.append( (char) c );
        }
    }

    /**
     * XML Element closing tag
     *
     * @param doc document content handler
     * @param c current char
     * @throws Exception parse error
     */
    private void closeTag( DocHandler doc, int c )
        throws Exception
    {
        if ( c == '>' )
        {
            mode = popMode( st );
            tagName = sb.toString();
            sb.setLength( 0 );
            depth--;
            if ( depth == 0 )
            {
                mode = DONE;
            }
            doc.endElement( tagName );
        }
        else
        {
            sb.append( (char) c );
        }
    }

    /**
     * Text node
     *
     * @param doc document content handler
     * @param c current char
     * @throws Exception parse error
     */
    private void textNode( DocHandler doc, int c )
        throws Exception
    {
        if ( c == '<' )
        {
            st.push( new Integer( mode ) );
            mode = START_TAG;
            if ( sb.length() > 0 )
            {
                doc.text( sb.toString() );
                sb.setLength( 0 );
            }
        }
        else if ( c == '&' )
        {
            st.push( new Integer( mode ) );
            mode = ENTITY;
            etag.setLength( 0 );
        }
        else
        {
            sb.append( (char) c );
        }
    }

    /**
     * Throws an exception with parse location
     *
     * @param s message
     * @throws Exception formatted exception
     */
    private void exc( String s )
        throws Exception
    {
        throw new Exception( s + " near line " + line + ", column " + col );
    }
}
