/*
 ~ Copyright 2006-2007 Nicolas De Loof.
 ~
 ~ Licensed under the Apache License, Version 2.0 (the "License");
 ~ you may not use this file except in compliance with the License.
 ~ You may obtain a copy of the License at
 ~
 ~      http://www.apache.org/licenses/LICENSE-2.0
 ~
 ~ Unless required by applicable law or agreed to in writing, software
 ~ distributed under the License is distributed on an "AS IS" BASIS,
 ~ WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 ~ See the License for the specific language governing permissions and
 ~ limitations under the License.
 */
package info.jmonit.config.runtime;

/**
 * This class uses reflexion to discover the runtime environment and the
 * available functionnalities, like Java5 runtime, JAXP support, third party
 * libraries present in classpath...
 * <p>
 * jMonit uses those informations to adapt itself to the environment, using
 * available tools to provide best functionnalities, and downgrading to
 * alternative code for minimal environments.
 *
 * @author <a href="mailto:ndeloof@sourceforge.net">ndeloof</a>
 */
public class RuntimeDiscovery
{
    /** Indicates Java 1.4 or better runtime environment */
    private static boolean java14 = isPresentInClasspath( "java.lang.StackTraceElement" );

    /** Indicates Java 5 or better runtime environment */
    private static boolean java5 = isPresentInClasspath( "java.util.concurrent.Executor" );

    /** Indicates support for the SAX xml parser API */
    private static boolean sax = isPresentInClasspath( "javax.xml.parsers.SAXParserFactory" );

    /** Indicates log4j present in classpath */
    private static boolean log4j = isPresentInClasspath( "org.apache.log4j.Logger" );

    /** Indicates jakarta commons-logging present in classpath */
    private static boolean commonsLogging = isPresentInClasspath( "org.apache.commons.logging.Log" );

    /** Indicates jakarta commons-collections present in classpath */
    private static boolean commonsCollections = isPresentInClasspath( "org.apache.commons.collections.FastTreeMap" );

    /** Indicates cglib present in classpath */
    private static boolean cglib = isPresentInClasspath( "net.sf.cglib.MethodInterceptor" );
    
    /**
     * @return true if java 1.4 is available
     */
    public static boolean isJava14()
    {
        return java14;
    }

    /**
     * @return true if java5 is available
     */
    public static boolean isJava5()
    {
        return java5;
    }

    /**
     * @return true if commons-collections is available
     */
    public static boolean isCommonsCollections()
    {
        return commonsCollections;
    }

    /**
     * @return true if commons-logging is available
     */
    public static boolean isCommonsLogging()
    {
        return commonsLogging;
    }

    /**
     * @return true if log4j is available
     */
    public static boolean isLog4j()
    {
        return log4j;
    }

    /**
     * @return the sax
     */
    public static boolean isSaxSupported()
    {
        return sax;
    }

    /**
     * @return the cglib
     */
    public static boolean isCglib()
    {
        return cglib;
    }

    /**
     * Check if the specified class is present in runtime classpath.
     *
     * @param name fully quelified class name
     * @return tru if class is available in runtime classpath.
     */
    private static boolean isPresentInClasspath( String name )
    {
        try
        {
            Class.forName( name );
            return true;
        }
        catch ( Exception e )
        {
            return false;
        }
    }

    /**
     * @param java14 the java14 to set
     */
    protected static void setJava14( boolean java14 )
    {
        RuntimeDiscovery.java14 = java14;
    }

    /**
     * @param java5 the java5 to set
     */
    protected static void setJava5( boolean java5 )
    {
        RuntimeDiscovery.java5 = java5;
    }

    /**
     * @param sax the sax to set
     */
    protected static void setSax( boolean sax )
    {
        RuntimeDiscovery.sax = sax;
    }

    /**
     * @param log4j the log4j to set
     */
    protected static void setLog4j( boolean log4j )
    {
        RuntimeDiscovery.log4j = log4j;
    }

    /**
     * @param commonsLogging the commonsLogging to set
     */
    protected static void setCommonsLogging( boolean commonsLogging )
    {
        RuntimeDiscovery.commonsLogging = commonsLogging;
    }

    /**
     *
     * @param commonsCollections the commonsCollections to set
     */
    protected static void setCommonsCollections( boolean commonsCollections )
    {
        RuntimeDiscovery.commonsCollections = commonsCollections;
    }

    /**
     * @param cglib the cglib to set
     */
    protected static void setCglib( boolean cglib )
    {
        RuntimeDiscovery.cglib = cglib;
    }
}
