/*
 ~ Copyright 2006-2007 Nicolas De Loof.
 ~
 ~ Licensed under the Apache License, Version 2.0 (the "License");
 ~ you may not use this file except in compliance with the License.
 ~ You may obtain a copy of the License at
 ~
 ~      http://www.apache.org/licenses/LICENSE-2.0
 ~
 ~ Unless required by applicable law or agreed to in writing, software
 ~ distributed under the License is distributed on an "AS IS" BASIS,
 ~ WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 ~ See the License for the specific language governing permissions and
 ~ limitations under the License.
 */
package info.jmonit.logger;

import java.io.PrintWriter;
import java.io.StringWriter;


/**
 * <code>Logger</code> implementation that write long to console (System.out).
 *
 * @author <a href="mailto:ndeloof@sourceforge.net">ndeloof</a>
 */
public class ConsoleLogger
    implements Logger
{
    /** No log at all */
    private static final int OFF = 1;

    /** Only errors */
    private static final int ERROR = 2;

    /** warnings */
    private static final int WARN = 3;

    /** Information messages */
    private static final int INFO = 4;

    /** debug */
    private static final int DEBUG = 5;

    /** All logs */
    private static final int ALL = 6;

    /** Logging class */
    private Class caller;

    /** System property for setting logging level */
    public static final String LEVEL = "jmonit.logger.level";

    /** Loging level */
    private static int level;

    static
    {
        String prop = System.getProperty( LEVEL );
        if ( prop != null )
        {
            level = Integer.parseInt( prop );
        }
    }

    /**
     * {@inheritDoc}
     *
     * @see info.jmonit.logger.Logger#isDebugEnabled()
     */
    public boolean isDebugEnabled()
    {
        return level >= DEBUG;
    }

    /**
     * Constructeur
     *
     * @param caller target logging class
     */
    public ConsoleLogger( Class caller )
    {
        super();
        this.caller = caller;
    }

    /**
     * {@inheritDoc}
     *
     * @see info.jmonit.logger.Logger#debug(java.lang.String)
     */
    public void debug( String message )
    {
        if ( level >= DEBUG )
        {
            log( DEBUG, message, null );
        }
    }

    /**
     * {@inheritDoc}
     *
     * @see info.jmonit.logger.Logger#info(java.lang.String)
     */
    public void info( String message )
    {
        if ( level >= INFO )
        {
            log( INFO, message, null );
        }
    }

    /**
     * {@inheritDoc}
     *
     * @see info.jmonit.logger.Logger#warn(java.lang.String)
     */
    public void warn( String message )
    {
        if ( level >= WARN )
        {
            log( WARN, message, null );
        }
    }

    /**
     * {@inheritDoc}
     *
     * @see info.jmonit.logger.Logger#warn(java.lang.String, java.lang.Exception)
     */
    public void warn( String message, Exception e )
    {
        if ( level >= WARN )
        {
            log( WARN, message, e );
        }
    }

    /**
     * {@inheritDoc}
     *
     * @see info.jmonit.logger.Logger#error(java.lang.String)
     */
    public void error( String message )
    {
        if ( level >= ERROR )
        {
            log( ERROR, message, null );
        }
    }

    /**
     * {@inheritDoc}
     *
     * @see info.jmonit.logger.Logger#error(java.lang.String, java.lang.Exception)
     */
    public void error( String message, Exception e )
    {
        if ( level >= ERROR )
        {
            log( ERROR, message, e );
        }
    }

    /**
     * Log a message
     * @param messageLevel message level
     * @param message message String
     * @param e exception to log (or null)
     */
    private void log( int messageLevel, String message, Exception e )
    {
        StringBuffer stb = new StringBuffer( "[" );
        switch ( messageLevel )
        {
            case DEBUG:
                stb.append( "DEBUG" );
                break;
            case INFO:
                stb.append( "INFO" );
                break;
            case WARN:
                stb.append( "WARN" );
                break;
            default:
                stb.append( "ERROR" );
                break;
        }
        stb.append( "] (" );
        stb.append( caller.getName() );
        stb.append( ") " );
        stb.append( message );
        if ( e != null )
        {
            StringWriter writer = new StringWriter();
            e.printStackTrace( new PrintWriter( writer ) );
            stb.append( "\n" );
            stb.append( writer.toString() );
        }

        if ( level < ERROR )
        {
            System.out.println( stb.toString() );
        }
        else
        {
            System.err.println( stb.toString() );
        }
    }
}
