/*
 ~ Copyright 2006-2007 Nicolas De Loof.
 ~
 ~ Licensed under the Apache License, Version 2.0 (the "License");
 ~ you may not use this file except in compliance with the License.
 ~ You may obtain a copy of the License at
 ~
 ~      http://www.apache.org/licenses/LICENSE-2.0
 ~
 ~ Unless required by applicable law or agreed to in writing, software
 ~ distributed under the License is distributed on an "AS IS" BASIS,
 ~ WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 ~ See the License for the specific language governing permissions and
 ~ limitations under the License.
 */
package info.jmonit.monitor;

import java.util.HashSet;
import java.util.Set;

import info.jmonit.Domain;
import info.jmonit.Repository;
import info.jmonit.config.DomainConfig;

/**
 * Abstract domain implementation with implementation for trivial <code>Domain</code>
 * methods.
 *
 * @author <a href="mailto:ndeloof@sourceforge.net">ndeloof</a>
 */
public abstract class AbstractDomain implements Domain
{

    /** domain name */
    private String name;

    /** Configuration for this domain */
    private DomainConfig configuration;

    /** Parent domain in domain hierarchy */
    private Domain parent;

    /** sub-domains */
    private Set children = new HashSet();

    /** Repository this domain belongs to */
    private Repository repository;

    /**
     * Constructor
     *
     * @param repository Repository this domain belongs to
     */
    public AbstractDomain( Repository repository )
    {
        super();
        this.repository = repository;
    }

    /**
     * {@inheritDoc}
     *
     * @see info.jmonit.Domain#getName()
     */
    public String getName()
    {
        return name;
    }

    /**
     * {@inheritDoc}
     *
     * @see info.jmonit.Domain#getConfiguration()
     */
    public DomainConfig getConfiguration()
    {
        return configuration;
    }

    /**
     * {@inheritDoc}
     *
     * @see info.jmonit.Domain#getParent()
     */
    public Domain getParent()
    {
        return parent;
    }

    /**
     * {@inheritDoc}
     *
     * @see info.jmonit.Domain#configure(info.jmonit.config.DomainConfig)
     */
    public void configure( DomainConfig domainConfig )
    {
        this.configuration = domainConfig;

        DomainConfig parentConfig;
        while ( ( parentConfig = domainConfig.getParentConfig() ) != null )
        {
            // Inherit parent configuration
            parentConfig.isEnabled();
        }
        // TODO Parse configuration to register extensions
    }


    /**
     * {@inheritDoc}
     *
     * @see info.jmonit.Domain#isEnabled()
     */
    public boolean isEnabled()
    {
        Boolean enabled = configuration.isEnabled();
        if ( enabled != null )
        {
            return enabled.booleanValue();
        }
        if ( parent != null )
        {
            return parent.isEnabled();
        }
        // By default
        return true;
    }

    /**
     * {@inheritDoc}
     *
     * @see info.jmonit.Domain#createSubDomain(java.lang.String)
     */
    public synchronized Domain createSubDomain( String childName )
    {
        String fullName = childName;
        if ( this.name != null )
        {
            fullName = name + '.' + childName;
        }
        AbstractDomain child = newChildDomainInstance();
        child.name = fullName;
        child.parent = this;
        children.add( child );
        repository.register( child );
        return child;
    }

    /**
     * Create an instance of AbstractDomain used as child for this domain.
     * @return new Concrete domain instance
     */
    protected abstract AbstractDomain newChildDomainInstance();

    /**
     * {@inheritDoc}
     *
     * @see java.lang.Comparable#compareTo(T)
     */
    public int compareTo( Object other )
    {
        if ( other instanceof Domain )
        {
            Domain d = (Domain) other;
            return name.compareTo( d.getName() );
        }
        return -1;
    }

    /**
     * @return the children
     */
    public synchronized  Set getChildren()
    {
        return new HashSet( children );
    }

    /**
     * @return the repository
     */
    protected Repository getRepository()
    {
        return repository;
    }
}