/*
 ~ Copyright 2006-2007 Nicolas De Loof.
 ~
 ~ Licensed under the Apache License, Version 2.0 (the "License");
 ~ you may not use this file except in compliance with the License.
 ~ You may obtain a copy of the License at
 ~
 ~      http://www.apache.org/licenses/LICENSE-2.0
 ~
 ~ Unless required by applicable law or agreed to in writing, software
 ~ distributed under the License is distributed on an "AS IS" BASIS,
 ~ WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 ~ See the License for the specific language governing permissions and
 ~ limitations under the License.
 */
package info.jmonit.monitor;

import info.jmonit.Agregator;
import info.jmonit.Domain;
import info.jmonit.Monitor;
import info.jmonit.Repository;
import info.jmonit.logger.Logger;
import info.jmonit.logger.LoggerFactory;
import info.jmonit.monitor.agregates.SimpleAgregator;
import info.jmonit.visitor.MonitorVisitor;
import info.jmonit.visitor.RepositoryVisitor;

import java.util.Collections;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Set;

/**
 * Default domain implementation. Uses datas from a DomainConfig instance build
 * by configuration parser to implement Domain.
 *
 * @author <a href="mailto:ndeloof@sourceforge.net">ndeloof</a>
 */
public class DefaultDomain extends AbstractDomain implements Domain
{
    /** Logger */
    private static Logger logger = LoggerFactory
            .getLogger( DefaultDomain.class );

    /** Monitors attached to this domain */
    private Map monitors = new HashMap();

    /**
     * Constructor
     * @param repository Repository this domain belongs to
     */
    public DefaultDomain( Repository repository )
    {
        super( repository );
    }

    /**
     * {@inheritDoc}
     *
     * @see info.jmonit.Domain#getMonitor(java.lang.String)
     */
    public synchronized Monitor getMonitor( String name )
    {
        Monitor monitor = (Monitor) monitors.get( name );
        if (monitor != null)
        {
            return monitor;
        }
        return createMonitor( name );
    }

    /**
     * {@inheritDoc}
     *
     * @see info.jmonit.Domain#createMonitor(java.lang.String)
     */
    public final Monitor createMonitor( String monitorName )
    {
        if ( logger.isDebugEnabled() )
        {
            logger.debug( "Creating monitor " + monitorName + " in domain "
                    + getName() );
        }
        Monitor monitor = newMonitorInstance( monitorName );
        monitors.put( monitorName, monitor );
        getRepository().register( monitor );
        return monitor;
    }

    /**
     * Create a monitor instance. May be overriden to use a custom Monitor class
     * or monitor creation strategy.
     *
     * @param monitorName
     *            monitor name
     * @return Monitor instance
     */
    protected Monitor newMonitorInstance( String monitorName )
    {
        DefaultMonitor monitor = new DefaultMonitor( this, monitorName );
        monitor.addAgregator( new SimpleAgregator( Agregator.DEFAULT ) );
        return monitor;
    }

    /**
     * {@inheritDoc}
     *
     * @see info.jmonit.monitor.AbstractDomain#newChildDomainInstance()
     */
    protected AbstractDomain newChildDomainInstance()
    {
        return new DefaultDomain( getRepository() );
    }

    /**
     * {@inheritDoc}
     *
     * @see info.jmonit.Domain#accept(RepositoryVisitor)
     */
    public void accept( RepositoryVisitor visitor )
    {
        if ( logger.isDebugEnabled() )
        {
            logger.debug( getName() + " accepts visitor " + visitor );
        }

        visitor.visit( this );

        if ( visitor instanceof MonitorVisitor )
        {
            MonitorVisitor monitorVisitor = (MonitorVisitor) visitor;
            // Visit monitors form this domain
            List visitables;
            synchronized ( monitors )
            {
                visitables = new LinkedList( monitors.values() );
            }
            Collections.sort( visitables );
            for ( Iterator iter = visitables.iterator(); iter.hasNext(); )
            {
                Monitor monitor = (Monitor) iter.next();
                monitor.accept( monitorVisitor );
            }
        }

        // Visit sub-domains
        Set children = getChildren();
        for ( Iterator iter = children.iterator(); iter.hasNext(); )
        {
            Domain child = (Domain) iter.next();
            child.accept( visitor );
        }

        visitor.visited( this );
    }
}
