/*
 ~ Copyright 2006-2007 Nicolas De Loof.
 ~
 ~ Licensed under the Apache License, Version 2.0 (the "License");
 ~ you may not use this file except in compliance with the License.
 ~ You may obtain a copy of the License at
 ~
 ~      http://www.apache.org/licenses/LICENSE-2.0
 ~
 ~ Unless required by applicable law or agreed to in writing, software
 ~ distributed under the License is distributed on an "AS IS" BASIS,
 ~ WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 ~ See the License for the specific language governing permissions and
 ~ limitations under the License.
 */
package info.jmonit.monitor;

import info.jmonit.Agregator;
import info.jmonit.Collector;
import info.jmonit.Domain;
import info.jmonit.Monitor;
import info.jmonit.Probe;
import info.jmonit.visitor.MonitorSnapshot;
import info.jmonit.visitor.MonitorVisitor;

import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

/**
 * Monitoring implementation that uses sychronized methods to ensure coherence
 * of agregated datas.
 *
 * @author <a href="mailto:ndeloof@sourceforge.net">ndeloof</a>
 */
public class DefaultMonitor implements Monitor, Collector
{
    /** Aggregator for datas gathered by the monitor */
    private Map agregators = new HashMap();

    /** Domain the monitor is attached to */
    private Domain domain;

    /** Monitoring name, unique inside it's domain */
    private String name;

    /**
     * Constructor
     *
     * @param domain
     *            Domain this monitor is attached to.
     * @param name
     *            Monitoring name.
     */
    public DefaultMonitor( Domain domain, String name )
    {
        super();
        this.name = name;
        this.domain = domain;
        clear();
    }

    /**
     * Register an aggregator for this monitor.
     * @param agregator
     *            to be registered
     */
    public synchronized void addAgregator( Agregator agregator )
    {
        agregators.put( agregator.getRole(), agregator );
    }

    /**
     * @param role
     *            role of the agregator for this monitor (feature)
     * @return agregator for the requested role
     */
    public synchronized Agregator getAgregator( String role )
    {
        return (Agregator) agregators.get( role );
    }

    /**
     * Unregister an aggregator for this monitor.
     *
     * @param agregator
     *            to be unregistered
     */
    public synchronized void removeAggregator( String role )
    {
        agregators.remove( role );
    }

    // --- Implementation of Monitor

    /**
     * This method implementation is not expected to be used as
     * InterceptorChainMonitor is the entry point to enable Interceptors. This
     * code is here only to honor the Monitor interface. {@inheritDoc}
     *
     * @see info.jmonit.Monitor#getProbe()
     */
    public Probe getProbe()
    {
        return new DefaultProbe( this );
    }

    /**
     *
     * {@inheritDoc}
     *
     * @see info.jmonit.Monitor#add(long)
     */
    public synchronized void add( long value )
    {
        for ( Iterator iter = agregators.values().iterator(); iter.hasNext(); )
        {
            Agregator agregator = (Agregator) iter.next();
            agregator.hit( System.currentTimeMillis() );
            agregator.addValue( value );
        }
    }

    /**
     * {@inheritDoc} Visit monitor and it's agregators, with special handling of
     * the default agregator that is responsible for creating the
     * MonitorSnapshot.
     *
     * @see info.jmonit.Monitor#accept(info.jmonit.visitor.MonitorVisitor)
     */
    public void accept( MonitorVisitor visitor )
    {
        visitor.visit( this );
        // Create a snapshot of monitor state.
        // This requires synchronized acces to the monitor to allow data
        // consistency.
        Map visitables;
        MonitorSnapshot snapshot = null;
        synchronized ( this )
        {
            visitables = new HashMap( agregators );
            Agregator agregator = (Agregator) agregators
                    .get( Agregator.DEFAULT );
            if ( agregator != null )
            {
                snapshot = new MonitorSnapshot( agregator );
            }
        }
        if ( snapshot != null )
        {
            visitor.visit( snapshot );
        }
        for ( Iterator iter = visitables.values().iterator(); iter.hasNext(); )
        {
            Agregator agregator = (Agregator) iter.next();
            agregator.accept( visitor );
        }
    }

    /**
     * {@inheritDoc}
     *
     * @see info.jmonit.Monitor#getDomain()
     */
    public Domain getDomain()
    {
        return domain;
    }

    /**
     * {@inheritDoc}
     *
     * @see info.jmonit.monitor.SnapshotSource#getName()
     */
    public String getName()
    {
        return name;
    }

    /**
     * {@inheritDoc}
     *
     * @see info.jmonit.Monitor#clear()
     */
    public synchronized void clear()
    {
        for ( Iterator iter = agregators.values().iterator(); iter.hasNext(); )
        {
            Agregator agregator = (Agregator) iter.next();
            agregator.clear();
        }
    }

    /**
     * {@inheritDoc}
     *
     * @see info.jmonit.Monitor#supports(java.lang.String)
     */
    public synchronized boolean supports( String feature )
    {
        return agregators.containsKey( feature );
    }

    // --- Implementation of Collector

    /**
     * {@inheritDoc}
     *
     * @see info.jmonit.Collector#started(Probe)
     */
    public synchronized void started( Probe probe )
    {
        for ( Iterator iter = agregators.values().iterator(); iter.hasNext(); )
        {
            Agregator agregator = (Agregator) iter.next();
            agregator.incrementActive();
        }
    }

    /**
     * {@inheritDoc}
     *
     * @see info.jmonit.Collector#stoped(info.jmonit.Probe,
     *      long, long)
     */
    public synchronized void stoped( Probe probe, long value, long time )
    {
        for ( Iterator iter = agregators.values().iterator(); iter.hasNext(); )
        {
            Agregator agregator = (Agregator) iter.next();
            agregator.hit( probe.startedAt() );
            agregator.decrementActive();
            agregator.addValue( value );
        }
    }

    /**
     * {@inheritDoc}
     *
     * @see info.jmonit.Collector#canceled(info.jmonit.Probe)
     */
    public synchronized void canceled( Probe probe )
    {
        for ( Iterator iter = agregators.values().iterator(); iter.hasNext(); )
        {
            Agregator agregator = (Agregator) iter.next();
            agregator.decrementActive();
        }
    }

    /**
     * {@inheritDoc}
     *
     * @override
     * @see java.lang.Comparable#compareTo(T)
     */
    public int compareTo( Object other )
    {
        if ( !( other instanceof Monitor ) ) { return -1; }
        Monitor m = (Monitor) other;
        int comp = getDomain().compareTo( m.getDomain() );
        if ( comp == 0 )
        {
            comp = m.getName().compareTo( m.getName() );
        }
        return comp;
    }

}
