/*
 ~ Copyright 2006-2007 Nicolas De Loof.
 ~
 ~ Licensed under the Apache License, Version 2.0 (the "License");
 ~ you may not use this file except in compliance with the License.
 ~ You may obtain a copy of the License at
 ~
 ~      http://www.apache.org/licenses/LICENSE-2.0
 ~
 ~ Unless required by applicable law or agreed to in writing, software
 ~ distributed under the License is distributed on an "AS IS" BASIS,
 ~ WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 ~ See the License for the specific language governing permissions and
 ~ limitations under the License.
 */
package info.jmonit.monitor;

import info.jmonit.Collector;
import info.jmonit.Probe;


/**
 * Simple probe implementation
 *
 * @author <a href="mailto:ndeloof@sourceforge.net">ndeloof</a>
 */
public class DefaultProbe
    implements Probe
{
    /** Time the probe was started */
    private long startedAt;

    /** Time the probe was stopped */
    private long stopedAt;

    /** Time the probe was paused */
    private long paused;

    /** flag for running probe */
    private boolean started;

    /** flag for stopped probe */
    private boolean stoped;

    /** Collector that is notified of probe events */
    private Collector collector;

    /**
     * Constructor
     *
     * @param collector Collector that is notified of probe events
     */
    public DefaultProbe( Collector collector )
    {
        super();
        this.collector = collector;
    }

    /**
     * {@inheritDoc}
     *
     * @see info.jmonit.Probe#start()
     */
    public long start()
    {
        if ( !started )
        {
            startedAt = System.currentTimeMillis();
            collector.started( this );
            started = true;
        }
        return startedAt;
    }

    /**
     * {@inheritDoc}
     *
     * @see info.jmonit.Probe#startedAt()
     */
    public long startedAt()
    {
        return startedAt;
    }

    /**
     * {@inheritDoc}
     *
     * @see info.jmonit.Probe#pause()
     */
    public void pause()
    {
        if ( started )
        {
            stopedAt = System.currentTimeMillis();
        }
    }

    /**
     * {@inheritDoc}
     *
     * @see info.jmonit.Probe#resume()
     */
    public void resume()
    {
        if ( stopedAt > 0 )
        {
            paused = System.currentTimeMillis() - stopedAt;
        }
        stopedAt = 0;
    }

    /**
     * {@inheritDoc}
     *
     * @see info.jmonit.Probe#stop()
     */
    public long stop()
    {
        if ( !stoped && started )
        {
            stopedAt = System.currentTimeMillis();
            collector.stoped( this, getElapsedTime(), stopedAt );
            stoped = true;
        }
        return stopedAt;
    }

    /**
     * {@inheritDoc}
     *
     * @see info.jmonit.Probe#cancel()
     */
    public void cancel()
    {
        if ( !stoped && started )
        {
            stoped = true;
            collector.canceled( this );
        }
    }

    /**
     * {@inheritDoc}
     *
     * @see info.jmonit.Probe#getElapsedTime()
     */
    public long getElapsedTime()
    {
        if ( stopedAt > 0 )
        {
            return stopedAt - startedAt - paused;
        }
        else
        {
            // Still running !
            return System.currentTimeMillis() - startedAt - paused;
        }
    }

    /**
     * @return the Collector.
     */
    public Collector getCollector()
    {
        return collector;
    }
}
