/*
 ~ Copyright 2006-2007 Nicolas De Loof.
 ~
 ~ Licensed under the Apache License, Version 2.0 (the "License");
 ~ you may not use this file except in compliance with the License.
 ~ You may obtain a copy of the License at
 ~
 ~      http://www.apache.org/licenses/LICENSE-2.0
 ~
 ~ Unless required by applicable law or agreed to in writing, software
 ~ distributed under the License is distributed on an "AS IS" BASIS,
 ~ WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 ~ See the License for the specific language governing permissions and
 ~ limitations under the License.
 */
package info.jmonit.monitor;

import info.jmonit.Domain;
import info.jmonit.Monitor;
import info.jmonit.Repository;
import info.jmonit.config.DomainConfig;
import info.jmonit.config.RepositoryConfig;
import info.jmonit.visitor.RepositoryVisitor;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

/**
 * @author <a href="mailto:ndeloof@sourceforge.net">ndeloof</a>
 */
public class DefaultRepository
    implements Repository
{
    /**
     * Initial size of monitors Map for better performances TODO : some bench
     * required
     */
    private static final int INITIAL_SIZE = 100;

    /** All monitors inside this repository for quick retrieval */
    private Map monitors = new HashMap( INITIAL_SIZE );

    /** All domains inside this repository for quick retrieval */
    private Map domains = new HashMap();

    /** Root of the domain hierarchy */
    private Domain rootDomain = new DefaultDomain( this );

    /**
     * {@inheritDoc}
     *
     * @see info.jmonit.Repository#getMonitor(java.lang.String,
     * java.lang.String)
     */
    public Monitor getMonitor( String domainName, String name )
    {
        Repository.Key key = new Repository.Key( domainName, name );
        // Find existing monitor or create a new one.
        synchronized ( monitors )
        {
            Monitor monitor = (Monitor) monitors.get( key );
            if ( monitor == null )
            {
                Domain domain = getDomain( domainName );
                monitor = domain.createMonitor( name );
            }
            return monitor;
        }
    }

    /**
     * {@inheritDoc}
     *
     * @see info.jmonit.monitor.domain.MonitorRegistrationCallBack#register(info.jmonit.Monitor)
     */
    public void register( Monitor monitor )
    {
        Repository.Key key = new Repository.Key( monitor.getDomain().getName(), monitor.getName() );
        monitors.put( key, monitor );
    }

    /**
     * {@inheritDoc}
     *
     * @see info.jmonit.Repository#getDomain(java.lang.String)
     */
    public synchronized Domain getDomain( final String name )
    {
        Domain domain = (Domain) domains.get( name );
        String localName = name;
        if ( domain == null )
        {
            Domain parent = rootDomain;
            int i = name.lastIndexOf( '.' );
            if ( i > 0 )
            {
                String parentName = name.substring( 0, i );
                parent = getDomain( parentName );
                localName = name.substring( i + 1 );
            }
            domain = parent.createSubDomain( localName );

        }
        return domain;
    }

    /**
     * {@inheritDoc}
     *
     * @see info.jmonit.monitor.domain.DomainRegistrationCallBack#register(info.jmonit.Domain)
     */
    public void register( Domain domain )
    {
        domains.put( domain.getName(), domain );
    }

    /**
     * {@inheritDoc}
     *
     * @see info.jmonit.Repository#accept(RepositoryVisitor)
     */
    public void accept( RepositoryVisitor visitor )
    {
        visitor.visit( this );
        rootDomain.accept( visitor );
    }

    /**
     * @param domain to be registered in this repository
     */
    public void registerDomain( Domain domain )
    {
        domains.put( domain.getName(), domain );
    }

    /**
     * Clear the repository
     */
    protected void clear()
    {
        monitors.clear();
    }

    /**
     * Configure the repository based on a set of DomainConfig.
     *
     * @param config repository configuration
     */
    public void configure( RepositoryConfig config )
    {
        Collection domainsConfig = config.getConfiguredDomains();

        // Order domain configs by name to assert parent domain will always
        // be configured before it's children
        List ordered = new ArrayList( domainsConfig );
        Collections.sort( ordered, new Comparator()
        {
            public int compare( Object arg0, Object arg1 )
            {
                DomainConfig d0 = (DomainConfig) arg0;
                DomainConfig d1 = (DomainConfig) arg1;
                return d0.getName().compareTo( d1.getName() );
            }
        } );

        // Create and configure domains
        for ( Iterator iter = ordered.iterator(); iter.hasNext(); )
        {
            DomainConfig domain = (DomainConfig) iter.next();
            String name = domain.getName();
            getDomain( name ).configure( domain );
        }
    }
}
