/*
 ~ Copyright 2006-2007 Nicolas De Loof.
 ~
 ~ Licensed under the Apache License, Version 2.0 (the "License");
 ~ you may not use this file except in compliance with the License.
 ~ You may obtain a copy of the License at
 ~
 ~      http://www.apache.org/licenses/LICENSE-2.0
 ~
 ~ Unless required by applicable law or agreed to in writing, software
 ~ distributed under the License is distributed on an "AS IS" BASIS,
 ~ WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 ~ See the License for the specific language governing permissions and
 ~ limitations under the License.
 */
package info.jmonit.monitor.agregates;

import info.jmonit.Agregator;
import info.jmonit.visitor.MonitorVisitor;

/**
 * Basic aggregator implementation, that aggregates monitored datas into
 * counters.
 *
 * @author <a href="mailto:ndeloof@sourceforge.net">ndeloof</a>
 */
public class SimpleAgregator implements Agregator
{
    /** Role of this agregator for the monitor */
    private String role;

    /** hits counter */
    private long hits;

    /** Total counter */
    private long total;

    /** Min value counter */
    private long min;

    /** Max value counter */
    private long max;

    /** Active threads counter */
    private long active;

    /** Max active thread counter */
    private long maxActive;

    /** First monitor use */
    private long firstUse;

    /** Last monitor use */
    private long lastUse;

    /**
     * Default Constructor
     *
     * @param role role (unique for a monitor)
     */
    public SimpleAgregator( String role )
    {
        super();
        this.role = role;
        clear();
    }

    /**
     * {@inheritDoc}
     * @override
     * @see info.jmonit.Agregator#hit(long)
     */
    public void hit( long time )
    {
        hits++;
        lastUse = time;
        if ( firstUse < 0 )
        {
            firstUse = time;
        }
    }

    /**
     * {@inheritDoc}
     *
     * @override
     * @see info.jmonit.Agregator#incrementActive()
     */
    public void incrementActive( )
    {
        active ++;
        if ( active > maxActive )
        {
            maxActive = active;
        }
    }

    /**
     * {@inheritDoc}
     *
     * @override
     * @see info.jmonit.Agregator#addValue(long)
     */
    public void addValue( long value )
    {
        total += value;
        if ( min > value )
        {
            min = value;
        }
        if ( max < value )
        {
            max = value;
        }
    }

    /**
     * {@inheritDoc}
     *
     * @override
     * @see info.jmonit.Agregator#decrementActive()
     */
    public void decrementActive( )
    {
        active --;
    }

    /**
     * {@inheritDoc}
     *
     * @override
     * @see info.jmonit.Agregator#clear()
     */
    public void clear()
    {
        hits = 0;
        total = 0;
        min = Long.MAX_VALUE;
        max = Long.MIN_VALUE;
        active = 0;
        maxActive = 0;
        firstUse = -1;
        lastUse = -1;
    }

    // --- SnapshotSource implementation ----------------------------------------------

    /**
     * @return the active
     */
    public long getActive()
    {
        return active;
    }

    /**
     * @return the firstUse
     */
    public long getFirstUse()
    {
        return firstUse;
    }

    /**
     * @return the hits
     */
    public long getHits()
    {
        return hits;
    }

    /**
     * @return the lastUse
     */
    public long getLastUse()
    {
        return lastUse;
    }

    /**
     * @return the max
     */
    public long getMax()
    {
        return max;
    }

    /**
     * @return the maxActive
     */
    public long getMaxActive()
    {
        return maxActive;
    }

    /**
     * @return the min
     */
    public long getMin()
    {
        return min;
    }

    /**
     * @return the total
     */
    public long getTotal()
    {
        return total;
    }


    /**
     * {@inheritDoc}
     * @see info.jmonit.monitor.SnapshotSource#getLoad()
     */
    public double getLoad()
    {
        return getLoad( lastUse - firstUse );
    }

    /**
     * @return The Average active counter.
     * @param period duration of the running period
     */
    protected double getLoad( long period )
    {
        if ( hits <= 1 )
        {
            return 0;
        }
        return ( (double) total ) / ( period );
    }

    /**
     * {@inheritDoc}
     *
     * @see info.jmonit.Agregator#accept(info.jmonit.visitor.MonitorVisitor)
     */
    public void accept( MonitorVisitor visitor )
    {
        visitor.visit( this );
    }

    /**
     * {@inheritDoc}
     * @return the role
     */
    public String getRole()
    {
        return role;
    }
}
