/*
 ~ Copyright 2006-2007 Nicolas De Loof.
 ~
 ~ Licensed under the Apache License, Version 2.0 (the "License");
 ~ you may not use this file except in compliance with the License.
 ~ You may obtain a copy of the License at
 ~
 ~      http://www.apache.org/licenses/LICENSE-2.0
 ~
 ~ Unless required by applicable law or agreed to in writing, software
 ~ distributed under the License is distributed on an "AS IS" BASIS,
 ~ WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 ~ See the License for the specific language governing permissions and
 ~ limitations under the License.
 */
package info.jmonit.support.jdbc;

import info.jmonit.Monitor;
import info.jmonit.Monitoring;

import java.sql.Connection;
import java.sql.Driver;
import java.sql.DriverManager;
import java.sql.DriverPropertyInfo;
import java.sql.SQLException;
import java.util.Properties;
import java.util.StringTokenizer;

/**
 * The JMonit driver allows to monitor JDBC connections to a database by simply
 * changing the DataSource / JDBC configuration.
 * <p>
 * A custom JDBC url protocol is used to attach this driver to monitored
 * database connections. To monitor a connection, simply change your connection
 * or datasource configuration to use :
 * <ul>
 * <li>Driver class name = "info.jmonit.support.soap.jdbc.JMonitDriver"</li>
 * <li>JDBC url = jdbc:<b>jmonit:driver=[<em>your JDBC driver class</em>]</b>:[<em>your JDBC driver URL</em>]</li>
 * </ul>
 * For example, to monitor a JDBC Oracle connection, use this JDBC url :
 *
 * <pre>
 *     jdbc:&lt;b&gt;jmonit:&lt;/b&gt;driver=oracle.jdbc.driver.OracleDriver:oracle:thin@://localhost:8000:myDataBase
 * </pre>
 *
 * <p>
 * jMonit can be used to monitor JDBC connections to multiple DataBases. It
 * detects use of multiple database when more than on jMonit JDBC URL is used
 * and automatically registers new jMonitDriver instances to handle them.
 *
 * @author <a href="mailto:ndeloof@sourceforge.net">ndeloof</a>
 */
public class JMonitDriver
    implements Driver
{
    /** property name for setting the JDBC monitoring domain name */
    private static final String DOMAIN_PROPERTY = "domain";

    /** property name for setting the real JDBC driver */
    private static final String DRIVER_PROPERTY = "driver";

    /** custom JDBC protocol string */
    private static final String JMONIT_JDBC_PROTOCOL = ":jmonit:";

    /** Real JDBC Driver used to connect to the Database */
    private Driver realDriver;

    /** Real JDBC URL used to connect to the Database */
    private String realUrl;
    
    /** jMonit domain for JDBC monitoring */
    private String domain = "jdbc";

    static
    {
        try
        {
            DriverManager.registerDriver( new JMonitDriver() );
        }
        catch ( SQLException e )
        {
            throw new ExceptionInInitializerError( "Failed to register jMonit JDBC Driver " + e );
        }
    }

    /**
     * Constructor
     */
    public JMonitDriver()
    {
        super();
    }

    /**
     * Accept URL that includes jMonit custom protocol.
     *
     * @see java.sql.Driver#acceptsURL(java.lang.String)
     * {@inheritDoc}
     */
    public synchronized boolean acceptsURL( String url )
        throws SQLException
    {
        int i = url.indexOf( JMONIT_JDBC_PROTOCOL );
        if ( i < 0 )
        {
            return false;
        }

        // Parse the URL to get real driver class name
        url = url.substring( i + JMONIT_JDBC_PROTOCOL.length() );
        i = url.indexOf( ':' );
        if ( i < 0 )
        {
            // Malformed URL
            throw new SQLException( 
                "Malformed URL : jMonit JDBC URL format is 'jdbc:jmonit:driver=xxx:<your JDBC URL here>'" );
        }

        String config = url.substring( 0, i );
        String targetUrl = "jdbc" + url.substring( i );
        if ( realUrl != null && !realUrl.equals( targetUrl ) )
        {
            // This instance is allready used for another database
            // Register a new instance to handle this new database URL
            DriverManager.registerDriver( new JMonitDriver() );
            return false;
        }
        realUrl = targetUrl;

        Properties properties = new Properties();
        StringTokenizer tokenizer = new StringTokenizer( config, ";" );
        while ( tokenizer.hasMoreTokens() )
        {
            String pair = tokenizer.nextToken().trim();
            i = pair.indexOf( '=' );
            if ( i < 0 )
            {
                continue;
            }
            String name = pair.substring( 0, i ).trim().toLowerCase();
            String value = pair.substring( i + 1 ).trim();
            properties.setProperty( name, value );
        }

        String driverClassName = properties.getProperty( DRIVER_PROPERTY );
        if ( properties.containsKey( DOMAIN_PROPERTY ) )
        {
            domain = properties.getProperty( DOMAIN_PROPERTY );
        }

        try
        {
            realDriver = (Driver) Class.forName( driverClassName ).newInstance();
        }
        catch ( Exception e )
        {
            throw new SQLException( "Failed to get instance of JDBC Driver " + driverClassName + " : " + e );
        }
        return true;
    }

    /**
     * @see java.sql.Driver#connect(java.lang.String, java.util.Properties)
     * {@inheritDoc}
     */
    public Connection connect( String url, Properties info )
        throws SQLException
    {
        if ( !acceptsURL( url ) )
        {
            return null;
        }

        Connection connection = realDriver.connect( realUrl, info );
        Monitor monitor = Monitoring.getMonitor( domain, "connections" );
        return MonitoredConnection.wrapp( connection, monitor );

    }

    /**
     * Delegates to the real driver.
     *
     * @see java.sql.Driver#getMajorVersion()
     * {@inheritDoc}
     */
    public int getMajorVersion()
    {
        return realDriver.getMajorVersion();
    }

    /**
     * Delegates to the real driver.
     *
     * @see java.sql.Driver#getMinorVersion()
     * {@inheritDoc}
     */
    public int getMinorVersion()
    {
        return realDriver.getMinorVersion();
    }

    /**
     * Delegates to the real driver.
     *
     * @see java.sql.Driver#getPropertyInfo(java.lang.String,
     *      java.util.Properties)
     * {@inheritDoc}
     */
    public DriverPropertyInfo[] getPropertyInfo( String url, Properties info )
        throws SQLException
    {
        return realDriver.getPropertyInfo( realUrl, info );
    }

    /**
     * Delegates to the real driver.
     *
     * @see java.sql.Driver#jdbcCompliant()
     * {@inheritDoc}
     */
    public boolean jdbcCompliant()
    {
        return realDriver.jdbcCompliant();
    }

}
