/*
 ~ Copyright 2006-2007 Nicolas De Loof.
 ~
 ~ Licensed under the Apache License, Version 2.0 (the "License");
 ~ you may not use this file except in compliance with the License.
 ~ You may obtain a copy of the License at
 ~
 ~      http://www.apache.org/licenses/LICENSE-2.0
 ~
 ~ Unless required by applicable law or agreed to in writing, software
 ~ distributed under the License is distributed on an "AS IS" BASIS,
 ~ WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 ~ See the License for the specific language governing permissions and
 ~ limitations under the License.
 */
package info.jmonit.support.jdbc;

import info.jmonit.Monitor;

import java.sql.CallableStatement;
import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.SQLException;
import java.sql.Savepoint;
import java.sql.Statement;

/**
 * JDBC 3.0 Monitored Connection. Fully implements Java 1.4 Connection API
 *
 * @author <a href="mailto:ndeloof@sourceforge.net">ndeloof</a>
 */
public class Jdbc3MonitoredConnection
    extends MonitoredConnection
{
    /**
     * @param connection target connection
     * @param monitor monitor for opened connexion
     */
    public Jdbc3MonitoredConnection( Connection connection, Monitor monitor )
    {
        super( connection, monitor );
    }

    /**
     * {@inheritDoc}
     *
     * @see java.sql.Connection#createStatement(int, int, int)
     */
    public Statement createStatement( int resultSetType, int resultSetConcurrency, int resultSetHoldability )
        throws SQLException
    {
        return getTargetConnection().createStatement( resultSetType, resultSetConcurrency, resultSetHoldability );
    }

    /**
     * {@inheritDoc}
     *
     * @see java.sql.Connection#getHoldability()
     */
    public int getHoldability()
        throws SQLException
    {
        return getTargetConnection().getHoldability();
    }

    /**
     * {@inheritDoc}
     *
     * @see java.sql.Connection#prepareCall(java.lang.String, int, int, int)
     */
    public CallableStatement prepareCall( String sql, int resultSetType, int resultSetConcurrency,
                                          int resultSetHoldability )
        throws SQLException
    {
        CallableStatement statement = getTargetConnection().prepareCall( sql, resultSetType, resultSetConcurrency,
                                                                         resultSetHoldability );
        return new Jdbc3MonitoredCallableStatement( statement, sql, this );
    }

    /**
     * {@inheritDoc}
     *
     * @see java.sql.Connection#prepareStatement(java.lang.String, int, int,
     * int)
     */
    public PreparedStatement prepareStatement( String sql, int resultSetType, int resultSetConcurrency,
                                               int resultSetHoldability )
        throws SQLException
    {
        PreparedStatement statement = getTargetConnection().prepareStatement( sql, resultSetType, resultSetConcurrency,
                                                                              resultSetHoldability );
        return new Jdbc3MonitoredPreparedStatement( statement, sql, this );
    }

    /**
     * {@inheritDoc}
     *
     * @see java.sql.Connection#prepareStatement(java.lang.String, int)
     */
    public PreparedStatement prepareStatement( String sql, int autoGeneratedKeys )
        throws SQLException
    {
        PreparedStatement statement = getTargetConnection().prepareStatement( sql, autoGeneratedKeys );
        return new Jdbc3MonitoredPreparedStatement( statement, sql, this );
    }

    /**
     * {@inheritDoc}
     *
     * @see java.sql.Connection#prepareStatement(java.lang.String, int[])
     */
    public PreparedStatement prepareStatement( String sql, int[] columnIndexes )
        throws SQLException
    {
        PreparedStatement statement = getTargetConnection().prepareStatement( sql, columnIndexes );
        return new Jdbc3MonitoredPreparedStatement( statement, sql, this );
    }

    /**
     * {@inheritDoc}
     *
     * @see java.sql.Connection#prepareStatement(java.lang.String,
     * java.lang.String[])
     */
    public PreparedStatement prepareStatement( String sql, String[] columnNames )
        throws SQLException
    {
        PreparedStatement statement = getTargetConnection().prepareStatement( sql, columnNames );
        return new Jdbc3MonitoredPreparedStatement( statement, sql, this );
    }

    /**
     * {@inheritDoc}
     *
     * @see java.sql.Connection#releaseSavepoint(java.sql.Savepoint)
     */
    public void releaseSavepoint( Savepoint savepoint )
        throws SQLException
    {
        getTargetConnection().releaseSavepoint( savepoint );
    }

    /**
     * {@inheritDoc}
     *
     * @see java.sql.Connection#rollback(java.sql.Savepoint)
     */
    public void rollback( Savepoint savepoint )
        throws SQLException
    {
        getTargetConnection().rollback( savepoint );
    }

    /**
     * {@inheritDoc}
     *
     * @see java.sql.Connection#setHoldability(int)
     */
    public void setHoldability( int holdability )
        throws SQLException
    {
        getTargetConnection().setHoldability( holdability );
    }

    /**
     * {@inheritDoc}
     *
     * @see java.sql.Connection#setSavepoint()
     */
    public Savepoint setSavepoint()
        throws SQLException
    {
        return getTargetConnection().setSavepoint();
    }

    /**
     * {@inheritDoc}
     *
     * @see java.sql.Connection#setSavepoint(java.lang.String)
     */
    public Savepoint setSavepoint( String name )
        throws SQLException
    {
        return getTargetConnection().setSavepoint( name );
    }

}
