/*
 ~ Copyright 2006-2007 Nicolas De Loof.
 ~
 ~ Licensed under the Apache License, Version 2.0 (the "License");
 ~ you may not use this file except in compliance with the License.
 ~ You may obtain a copy of the License at
 ~
 ~      http://www.apache.org/licenses/LICENSE-2.0
 ~
 ~ Unless required by applicable law or agreed to in writing, software
 ~ distributed under the License is distributed on an "AS IS" BASIS,
 ~ WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 ~ See the License for the specific language governing permissions and
 ~ limitations under the License.
 */
package info.jmonit.support.jdbc;

import info.jmonit.Monitoring;
import info.jmonit.Probe;

import java.io.InputStream;
import java.io.Reader;
import java.math.BigDecimal;
import java.net.URL;
import java.sql.Array;
import java.sql.Blob;
import java.sql.CallableStatement;
import java.sql.Clob;
import java.sql.Date;
import java.sql.ParameterMetaData;
import java.sql.PreparedStatement;
import java.sql.Ref;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.sql.Time;
import java.sql.Timestamp;
import java.util.Calendar;
import java.util.Map;

/**
 * Wrapper for JDBC statements. Implements JDBC 3.0 methods
 *
 * @author <a href="mailto:ndeloof@sourceforge.net">ndeloof</a>
 */
public class Jdbc3MonitoredStatement
    extends MonitoredStatement
{

    /** delegate statement */
    private Statement statement;

    /** delegate preparedStatement */
    private PreparedStatement preparedStatement;

    /** delegate callableStatement */
    private CallableStatement callableStatement;

    /**
     * @param statement target statement
     * @param connection monitored connection
     */
    public Jdbc3MonitoredStatement( Statement statement, MonitoredConnection connection )
    {
        super( statement, connection );
        this.statement = statement;
    }

    /**
     * @param statement target statement
     * @param query SQL query
     * @param connection monitored connection
     */
    public Jdbc3MonitoredStatement( PreparedStatement statement, String query, MonitoredConnection connection )
    {
        super( statement, query, connection );
        this.statement = statement;
        this.preparedStatement = statement;
    }

    /**
     * @param statement target statement
     * @param query SQL query
     * @param connection monitored connection
     */
    public Jdbc3MonitoredStatement( CallableStatement statement, String query, MonitoredConnection connection )
    {
        super( statement, query, connection );
        this.statement = statement;
        this.preparedStatement = statement;
        this.callableStatement = statement;
    }

    /**
     * {@inheritDoc}
     *
     * @see java.sql.Statement#execute(java.lang.String, int)
     */
    public boolean execute( String sql, int autoGeneratedKeys )
        throws SQLException
    {
        Probe probe = Monitoring.start( "jdbc", generalizeSql( sql ) );
        try
        {
            return statement.execute( sql, autoGeneratedKeys );
        }
        catch ( SQLException sqle )
        {
            throw handleException( sqle );
        }
        finally
        {
            probe.stop();
        }
    }

    /**
     * {@inheritDoc}
     *
     * @see java.sql.Statement#execute(java.lang.String, int[])
     */
    public boolean execute( String sql, int[] columnIndexes )
        throws SQLException
    {
        Probe probe = Monitoring.start( "jdbc", generalizeSql( sql ) );
        try
        {
            return statement.execute( sql, columnIndexes );
        }
        catch ( SQLException sqle )
        {
            throw handleException( sqle );
        }
        finally
        {
            probe.stop();
        }
    }

    /**
     * {@inheritDoc}
     *
     * @see java.sql.Statement#execute(java.lang.String, java.lang.String[])
     */
    public boolean execute( String sql, String[] columnNames )
        throws SQLException
    {
        Probe probe = Monitoring.start( "jdbc", generalizeSql( sql ) );
        try
        {
            return statement.execute( sql, columnNames );
        }
        catch ( SQLException sqle )
        {
            throw handleException( sqle );
        }
        finally
        {
            probe.stop();
        }
    }

    /**
     * {@inheritDoc}
     *
     * @see java.sql.Statement#executeUpdate(java.lang.String, int)
     */
    public int executeUpdate( String sql, int autoGeneratedKeys )
        throws SQLException
    {
        Probe probe = Monitoring.start( "jdbc", generalizeSql( sql ) );
        try
        {
            return statement.executeUpdate( sql, autoGeneratedKeys );
        }
        catch ( SQLException sqle )
        {
            throw handleException( sqle );
        }
        finally
        {
            probe.stop();
        }
    }

    /**
     * {@inheritDoc}
     *
     * @see java.sql.Statement#executeUpdate(java.lang.String, int[])
     */
    public int executeUpdate( String sql, int[] columnIndexes )
        throws SQLException
    {
        Probe probe = Monitoring.start( "jdbc", generalizeSql( sql ) );
        try
        {
            return statement.executeUpdate( sql, columnIndexes );
        }
        catch ( SQLException sqle )
        {
            throw handleException( sqle );
        }
        finally
        {
            probe.stop();
        }
    }

    /**
     * {@inheritDoc}
     *
     * @see java.sql.Statement#executeUpdate(java.lang.String,
     * java.lang.String[])
     */
    public int executeUpdate( String sql, String[] columnNames )
        throws SQLException
    {
        Probe probe = Monitoring.start( "jdbc", generalizeSql( sql ) );
        try
        {
            return statement.executeUpdate( sql, columnNames );
        }
        catch ( SQLException sqle )
        {
            throw handleException( sqle );
        }
        finally
        {
            probe.stop();
        }
    }

    /**
     * {@inheritDoc}
     *
     * @see java.sql.Statement#getGeneratedKeys()
     */
    public ResultSet getGeneratedKeys()
        throws SQLException
    {
        return statement.getGeneratedKeys();
    }

    /**
     * {@inheritDoc}
     *
     * @see java.sql.Statement#getMoreResults(int)
     */
    public boolean getMoreResults( int current )
        throws SQLException
    {
        return statement.getMoreResults( current );
    }

    /**
     * {@inheritDoc}
     *
     * @see java.sql.Statement#getResultSetHoldability()
     */
    public int getResultSetHoldability()
        throws SQLException
    {
        return statement.getResultSetHoldability();
    }

    /**
     * {@inheritDoc}
     *
     * @see java.sql.PreparedStatement#getParameterMetaData()
     */
    public ParameterMetaData getParameterMetaData()
        throws SQLException
    {
        return preparedStatement.getParameterMetaData();
    }

    /**
     * {@inheritDoc}
     *
     * @see java.sql.PreparedStatement#setURL(int, java.net.URL)
     */
    public void setURL( int parameterIndex, URL x )
        throws SQLException
    {
        preparedStatement.setURL( parameterIndex, x );
    }

    /**
     * {@inheritDoc}
     * @see java.sql.CallableStatement#getArray(java.lang.String)
     */
    public Array getArray( String parameterName )
        throws SQLException
    {
        return callableStatement.getArray( parameterName );
    }

    /**
     * {@inheritDoc}
     * @see java.sql.CallableStatement#getBigDecimal(java.lang.String)
     */
    public BigDecimal getBigDecimal( String parameterName )
        throws SQLException
    {
        return callableStatement.getBigDecimal( parameterName );
    }

    /**
     * {@inheritDoc}
     * @see java.sql.CallableStatement#getBlob(java.lang.String)
     */
    public Blob getBlob( String parameterName )
        throws SQLException
    {
        return callableStatement.getBlob( parameterName );
    }

    /**
     * {@inheritDoc}
     * @see java.sql.CallableStatement#getBoolean(java.lang.String)
     */
    public boolean getBoolean( String parameterName )
        throws SQLException
    {
        return callableStatement.getBoolean( parameterName );
    }

    /**
     * {@inheritDoc}
     * @see java.sql.CallableStatement#getByte(java.lang.String)
     */
    public byte getByte( String parameterName )
        throws SQLException
    {
        return callableStatement.getByte( parameterName );
    }

    /**
     * {@inheritDoc}
     * @see java.sql.CallableStatement#getBytes(java.lang.String)
     */
    public byte[] getBytes( String parameterName )
        throws SQLException
    {
        return callableStatement.getBytes( parameterName );
    }

    /**
     * {@inheritDoc}
     * @see java.sql.CallableStatement#getClob(java.lang.String)
     */
    public Clob getClob( String parameterName )
        throws SQLException
    {
        return callableStatement.getClob( parameterName );
    }

    /**
     * {@inheritDoc}
     * @see java.sql.CallableStatement#getDate(java.lang.String, java.util.Calendar)
     */
    public Date getDate( String parameterName, Calendar cal )
        throws SQLException
    {
        return callableStatement.getDate( parameterName, cal );
    }

    /**
     * {@inheritDoc}
     * @see java.sql.CallableStatement#getDate(java.lang.String)
     */
    public Date getDate( String parameterName )
        throws SQLException
    {
        return callableStatement.getDate( parameterName );
    }

    /**
     * {@inheritDoc}
     * @see java.sql.CallableStatement#getDouble(java.lang.String)
     */
    public double getDouble( String parameterName )
        throws SQLException
    {
        return callableStatement.getDouble( parameterName );
    }

    /**
     * {@inheritDoc}
     * @see java.sql.CallableStatement#getFloat(java.lang.String)
     */
    public float getFloat( String parameterName )
        throws SQLException
    {
        return callableStatement.getFloat( parameterName );
    }

    /**
     * {@inheritDoc}
     * @see java.sql.CallableStatement#getInt(java.lang.String)
     */
    public int getInt( String parameterName )
        throws SQLException
    {
        return callableStatement.getInt( parameterName );
    }

    /**
     * {@inheritDoc}
     * @see java.sql.CallableStatement#getLong(java.lang.String)
     */
    public long getLong( String parameterName )
        throws SQLException
    {
        return callableStatement.getLong( parameterName );
    }

    /**
     * {@inheritDoc}
     * @see java.sql.CallableStatement#getObject(java.lang.String, java.util.Map)
     */
    public Object getObject( String arg0, Map arg1 )
        throws SQLException
    {
        return callableStatement.getObject( arg0, arg1 );
    }

    /**
     * {@inheritDoc}
     * @see java.sql.CallableStatement#getObject(java.lang.String)
     */
    public Object getObject( String parameterName )
        throws SQLException
    {
        return callableStatement.getObject( parameterName );
    }

    /**
     * {@inheritDoc}
     * @see java.sql.CallableStatement#getRef(java.lang.String)
     */
    public Ref getRef( String parameterName )
        throws SQLException
    {
        return callableStatement.getRef( parameterName );
    }

    /**
     * {@inheritDoc}
     * @see java.sql.CallableStatement#getShort(java.lang.String)
     */
    public short getShort( String parameterName )
        throws SQLException
    {
        return callableStatement.getShort( parameterName );
    }

    /**
     * {@inheritDoc}
     * @see java.sql.CallableStatement#getString(java.lang.String)
     */
    public String getString( String parameterName )
        throws SQLException
    {
        return callableStatement.getString( parameterName );
    }

    /**
     * {@inheritDoc}
     * @see java.sql.CallableStatement#getTime(java.lang.String, java.util.Calendar)
     */
    public Time getTime( String parameterName, Calendar cal )
        throws SQLException
    {
        return callableStatement.getTime( parameterName, cal );
    }

    /**
     * {@inheritDoc}
     * @see java.sql.CallableStatement#getTime(java.lang.String)
     */
    public Time getTime( String parameterName )
        throws SQLException
    {
        return callableStatement.getTime( parameterName );
    }

    /**
     * {@inheritDoc}
     * @see java.sql.CallableStatement#getTimestamp(java.lang.String, java.util.Calendar)
     */
    public Timestamp getTimestamp( String parameterName, Calendar cal )
        throws SQLException
    {
        return callableStatement.getTimestamp( parameterName, cal );
    }

    /**
     * {@inheritDoc}
     * @see java.sql.CallableStatement#getTimestamp(java.lang.String)
     */
    public Timestamp getTimestamp( String parameterName )
        throws SQLException
    {
        return callableStatement.getTimestamp( parameterName );
    }

    /**
     * {@inheritDoc}
     * @see java.sql.CallableStatement#getURL(int)
     */
    public URL getURL( int parameterIndex )
        throws SQLException
    {
        return callableStatement.getURL( parameterIndex );
    }

    /**
     * {@inheritDoc}
     * @see java.sql.CallableStatement#getURL(java.lang.String)
     */
    public URL getURL( String parameterName )
        throws SQLException
    {
        return callableStatement.getURL( parameterName );
    }

    /**
     * {@inheritDoc}
     * @see java.sql.CallableStatement#registerOutParameter(java.lang.String, int, int)
     */
    public void registerOutParameter( String parameterName, int sqlType, int scale )
        throws SQLException
    {
        callableStatement.registerOutParameter( parameterName, sqlType, scale );
    }

    /**
     * {@inheritDoc}
     * @see java.sql.CallableStatement#registerOutParameter(java.lang.String, int, java.lang.String)
     */
    public void registerOutParameter( String parameterName, int sqlType, String typeName )
        throws SQLException
    {
        callableStatement.registerOutParameter( parameterName, sqlType, typeName );
    }

    /**
     * {@inheritDoc}
     * @see java.sql.CallableStatement#registerOutParameter(java.lang.String, int)
     */
    public void registerOutParameter( String parameterName, int sqlType )
        throws SQLException
    {
        callableStatement.registerOutParameter( parameterName, sqlType );
    }

    /**
     * {@inheritDoc}
     * @see java.sql.CallableStatement#setAsciiStream(java.lang.String, java.io.InputStream, int)
     */
    public void setAsciiStream( String parameterName, InputStream x, int length )
        throws SQLException
    {
        callableStatement.setAsciiStream( parameterName, x, length );
    }

    /**
     * {@inheritDoc}
     * @see java.sql.CallableStatement#setBigDecimal(java.lang.String, java.math.BigDecimal)
     */
    public void setBigDecimal( String parameterName, BigDecimal x )
        throws SQLException
    {
        callableStatement.setBigDecimal( parameterName, x );
    }

    /**
     * {@inheritDoc}
     * @see java.sql.CallableStatement#setBinaryStream(java.lang.String, java.io.InputStream, int)
     */
    public void setBinaryStream( String parameterName, InputStream x, int length )
        throws SQLException
    {
        callableStatement.setBinaryStream( parameterName, x, length );
    }

    /**
     * {@inheritDoc}
     * @see java.sql.CallableStatement#setBoolean(java.lang.String, boolean)
     */
    public void setBoolean( String parameterName, boolean x )
        throws SQLException
    {
        callableStatement.setBoolean( parameterName, x );
    }

    /**
     * {@inheritDoc}
     * @see java.sql.CallableStatement#setByte(java.lang.String, byte)
     */
    public void setByte( String parameterName, byte x )
        throws SQLException
    {
        callableStatement.setByte( parameterName, x );
    }

    /**
     * {@inheritDoc}
     * @see java.sql.CallableStatement#setBytes(java.lang.String, byte[])
     */
    public void setBytes( String parameterName, byte[] x )
        throws SQLException
    {
        callableStatement.setBytes( parameterName, x );
    }

    /**
     * {@inheritDoc}
     * @see java.sql.CallableStatement#setCharacterStream(java.lang.String, java.io.Reader, int)
     */
    public void setCharacterStream( String parameterName, Reader reader, int length )
        throws SQLException
    {
        callableStatement.setCharacterStream( parameterName, reader, length );
    }

    /**
     * {@inheritDoc}
     * @see java.sql.CallableStatement#setDate(java.lang.String, java.sql.Date, java.util.Calendar)
     */
    public void setDate( String parameterName, Date x, Calendar cal )
        throws SQLException
    {
        callableStatement.setDate( parameterName, x, cal );
    }

    /**
     * {@inheritDoc}
     * @see java.sql.CallableStatement#setDouble(java.lang.String, double)
     */
    public void setDouble( String parameterName, double x )
        throws SQLException
    {
        callableStatement.setDouble( parameterName, x );
    }

    /**
     * {@inheritDoc}
     * @see java.sql.CallableStatement#setFloat(java.lang.String, float)
     */
    public void setFloat( String parameterName, float x )
        throws SQLException
    {
        callableStatement.setFloat( parameterName, x );
    }

    /**
     * {@inheritDoc}
     * @see java.sql.CallableStatement#setInt(java.lang.String, int)
     */
    public void setInt( String parameterName, int x )
        throws SQLException
    {
        callableStatement.setInt( parameterName, x );
    }

    /**
     * {@inheritDoc}
     * @see java.sql.CallableStatement#setLong(java.lang.String, long)
     */
    public void setLong( String parameterName, long x )
        throws SQLException
    {
        callableStatement.setLong( parameterName, x );
    }

    /**
     * {@inheritDoc}
     * @see java.sql.CallableStatement#setNull(java.lang.String, int, java.lang.String)
     */
    public void setNull( String parameterName, int sqlType, String typeName )
        throws SQLException
    {
        callableStatement.setNull( parameterName, sqlType, typeName );
    }

    /**
     * {@inheritDoc}
     * @see java.sql.CallableStatement#setNull(java.lang.String, int)
     */
    public void setNull( String parameterName, int sqlType )
        throws SQLException
    {
        callableStatement.setNull( parameterName, sqlType );
    }

    /**
     * {@inheritDoc}
     * @see java.sql.CallableStatement#setObject(java.lang.String, java.lang.Object, int, int)
     */
    public void setObject( String parameterName, Object x, int targetSqlType, int scale )
        throws SQLException
    {
        callableStatement.setObject( parameterName, x, targetSqlType, scale );
    }

    /**
     * {@inheritDoc}
     * @see java.sql.CallableStatement#setObject(java.lang.String, java.lang.Object, int)
     */
    public void setObject( String parameterName, Object x, int targetSqlType )
        throws SQLException
    {
        callableStatement.setObject( parameterName, x, targetSqlType );
    }

    /**
     * {@inheritDoc}
     * @see java.sql.CallableStatement#setObject(java.lang.String, java.lang.Object)
     */
    public void setObject( String parameterName, Object x )
        throws SQLException
    {
        callableStatement.setObject( parameterName, x );
    }

    /**
     * {@inheritDoc}
     * @see java.sql.CallableStatement#setShort(java.lang.String, short)
     */
    public void setShort( String parameterName, short x )
        throws SQLException
    {
        callableStatement.setShort( parameterName, x );
    }

    /**
     * {@inheritDoc}
     * @see java.sql.CallableStatement#setString(java.lang.String, java.lang.String)
     */
    public void setString( String parameterName, String x )
        throws SQLException
    {
        callableStatement.setString( parameterName, x );
    }

    /**
     * {@inheritDoc}
     * @see java.sql.CallableStatement#setTime(java.lang.String, java.sql.Time, java.util.Calendar)
     */
    public void setTime( String parameterName, Time x, Calendar cal )
        throws SQLException
    {
        callableStatement.setTime( parameterName, x, cal );
    }

    /**
     * {@inheritDoc}
     * @see java.sql.CallableStatement#setTime(java.lang.String, java.sql.Time)
     */
    public void setTime( String parameterName, Time x )
        throws SQLException
    {
        callableStatement.setTime( parameterName, x );
    }

    /**
     * {@inheritDoc}
     * @see java.sql.CallableStatement#setTimestamp(java.lang.String, java.sql.Timestamp, java.util.Calendar)
     */
    public void setTimestamp( String parameterName, Timestamp x, Calendar cal )
        throws SQLException
    {
        callableStatement.setTimestamp( parameterName, x, cal );
    }

    /**
     * {@inheritDoc}
     * @see java.sql.CallableStatement#setTimestamp(java.lang.String, java.sql.Timestamp)
     */
    public void setTimestamp( String parameterName, Timestamp x )
        throws SQLException
    {
        callableStatement.setTimestamp( parameterName, x );
    }

    /**
     * {@inheritDoc}
     * @see java.sql.CallableStatement#setURL(java.lang.String, java.net.URL)
     */
    public void setURL( String parameterName, URL val )
        throws SQLException
    {
        callableStatement.setURL( parameterName, val );
    }

    /**
     * {@inheritDoc}
     * @see java.sql.CallableStatement#setDate(java.lang.String, java.sql.Date)
     */
    public void setDate( String parameterName, Date x )
        throws SQLException
    {
        callableStatement.setDate( parameterName, x );
    }
}
