/*
 ~ Copyright 2006-2007 Nicolas De Loof.
 ~
 ~ Licensed under the Apache License, Version 2.0 (the "License");
 ~ you may not use this file except in compliance with the License.
 ~ You may obtain a copy of the License at
 ~
 ~      http://www.apache.org/licenses/LICENSE-2.0
 ~
 ~ Unless required by applicable law or agreed to in writing, software
 ~ distributed under the License is distributed on an "AS IS" BASIS,
 ~ WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 ~ See the License for the specific language governing permissions and
 ~ limitations under the License.
 */
package info.jmonit.support.jdbc;

import info.jmonit.Monitor;
import info.jmonit.Probe;
import info.jmonit.config.runtime.FactorySupport;
import info.jmonit.config.runtime.RuntimeDiscovery;

import java.sql.CallableStatement;
import java.sql.Connection;
import java.sql.DatabaseMetaData;
import java.sql.PreparedStatement;
import java.sql.SQLException;
import java.sql.SQLWarning;
import java.sql.Statement;
import java.util.Map;

/**
 * Wrapper around a JDBC connection. Only implements Java 1.3 JDBC-2.0 api to be fully
 * compatible with Java 1.3 runtime.
 *
 * @author <a href="mailto:ndeloof@sourceforge.net">ndeloof</a>
 */
public abstract class MonitoredConnection
    implements Connection
{
    /** Implementation class for Connection when using JDBC3 */
    private static final String JDBC3_CONNECTION = "info.jmonit.support.jdbc.Jdbc3MonitoredConnection";

    /** Implementation class for Statement when using JDBC3 */
    private static final String JDBC3_STATEMENT = "info.jmonit.support.jdbc.Jdbc3MonitoredStatement";

    /** Implementation class for PreparedStatement when using JDBC3 */
    private static final String JDBC3_PREPARED_STATEMENT = "info.jmonit.support.jdbc.Jdbc3MonitoredPreparedStatement";

    /** Implementation class for CallableStatement when using JDBC3 */
    private static final String JDBC3_CALLABLE_STATEMENT = "info.jmonit.support.jdbc.Jdbc3MonitoredCallableStatement";


    /** target connection */
    private Connection connection;

    /** Probe for this connection */
    private Probe connectionOpenProbe;
    
    /** jMonit domain for JDBC monitoring */
    private String domain;

    /**
     * @param connection target connection
     * @param monitor monitor for opened connections
     */
    public MonitoredConnection( Connection connection, Monitor monitor )
    {
        super();
        this.connection = connection;
        connectionOpenProbe = monitor.getProbe();
        connectionOpenProbe.start();
        this.domain = monitor.getDomain().getName();
    }

    /**
     * {@inheritDoc}
     * @see java.sql.Connection#clearWarnings()
     */
    public void clearWarnings()
        throws SQLException
    {
        connection.clearWarnings();
    }

    /**
     * {@inheritDoc}
     * @see java.sql.Connection#close()
     */
    public void close()
        throws SQLException
    {
        connection.close();
        connectionOpenProbe.stop();
    }

    /**
     * {@inheritDoc}
     * @see java.sql.Connection#commit()
     */
    public void commit()
        throws SQLException
    {
        connection.commit();
    }

    /**
     * {@inheritDoc}
     * @see java.sql.Connection#createStatement()
     */
    public Statement createStatement()
        throws SQLException
    {
        Statement statement = connection.createStatement();
        return wrapp( statement );
    }

    /**
     * {@inheritDoc}
     * @see java.sql.Connection#createStatement(int, int)
     */
    public Statement createStatement( int resultSetType, int resultSetConcurrency )
        throws SQLException
    {
        Statement statement = connection.createStatement( resultSetType, resultSetConcurrency );
        return wrapp( statement );
    }

    /**
     * {@inheritDoc}
     * @see java.sql.Connection#getAutoCommit()
     */
    public boolean getAutoCommit()
        throws SQLException
    {
        return connection.getAutoCommit();
    }

    /**
     * {@inheritDoc}
     * @see java.sql.Connection#getCatalog()
     */
    public String getCatalog()
        throws SQLException
    {
        return connection.getCatalog();
    }

    /**
     * {@inheritDoc}
     * @see java.sql.Connection#getMetaData()
     */
    public DatabaseMetaData getMetaData()
        throws SQLException
    {
        return connection.getMetaData();
    }

    /**
     * {@inheritDoc}
     * @see java.sql.Connection#getTransactionIsolation()
     */
    public int getTransactionIsolation()
        throws SQLException
    {
        return connection.getTransactionIsolation();
    }

    /**
     * {@inheritDoc}
     * @see java.sql.Connection#getTypeMap()
     */
    public Map getTypeMap()
        throws SQLException
    {
        return connection.getTypeMap();
    }

    /**
     * {@inheritDoc}
     * @see java.sql.Connection#getWarnings()
     */
    public SQLWarning getWarnings()
        throws SQLException
    {
        return connection.getWarnings();
    }

    /**
     * {@inheritDoc}
     * @see java.sql.Connection#isClosed()
     */
    public boolean isClosed()
        throws SQLException
    {
        return connection.isClosed();
    }

    /**
     * {@inheritDoc}
     * @see java.sql.Connection#isReadOnly()
     */
    public boolean isReadOnly()
        throws SQLException
    {
        return connection.isReadOnly();
    }

    /**
     * {@inheritDoc}
     * @see java.sql.Connection#nativeSQL(java.lang.String)
     */
    public String nativeSQL( String sql )
        throws SQLException
    {
        return connection.nativeSQL( sql );
    }

    /**
     * {@inheritDoc}
     * @see java.sql.Connection#prepareCall(java.lang.String, int, int)
     */
    public CallableStatement prepareCall( String sql, int resultSetType, int resultSetConcurrency )
        throws SQLException
    {
        CallableStatement statement = connection.prepareCall( sql, resultSetType, resultSetConcurrency );
        return wrapp( statement, sql );
    }

    /**
     * {@inheritDoc}
     * @see java.sql.Connection#prepareCall(java.lang.String)
     */
    public CallableStatement prepareCall( String sql )
        throws SQLException
    {
        CallableStatement statement = connection.prepareCall( sql );
        return wrapp( statement, sql );
    }

    /**
     * {@inheritDoc}
     * @see java.sql.Connection#prepareStatement(java.lang.String, int, int)
     */
    public PreparedStatement prepareStatement( String sql, int resultSetType, int resultSetConcurrency )
        throws SQLException
    {
        PreparedStatement statement = connection.prepareStatement( sql, resultSetType, resultSetConcurrency );
        return wrapp( statement, sql );
    }

    /**
     * {@inheritDoc}
     * @see java.sql.Connection#prepareStatement(java.lang.String)
     */
    public PreparedStatement prepareStatement( String sql )
        throws SQLException
    {
        PreparedStatement statement =  connection.prepareStatement( sql );
        return wrapp( statement, sql );
    }

    /**
     * {@inheritDoc}
     * @see java.sql.Connection#rollback()
     */
    public void rollback()
        throws SQLException
    {
        connection.rollback();
    }

    /**
     * {@inheritDoc}
     * @see java.sql.Connection#setAutoCommit(boolean)
     */
    public void setAutoCommit( boolean autoCommit )
        throws SQLException
    {
        connection.setAutoCommit( autoCommit );
    }

    /**
     * {@inheritDoc}
     * @see java.sql.Connection#setCatalog(java.lang.String)
     */
    public void setCatalog( String catalog )
        throws SQLException
    {
        connection.setCatalog( catalog );
    }

    /**
     * {@inheritDoc}
     * @see java.sql.Connection#setReadOnly(boolean)
     */
    public void setReadOnly( boolean readOnly )
        throws SQLException
    {
        connection.setReadOnly( readOnly );
    }

    /**
     * {@inheritDoc}
     * @see java.sql.Connection#setTransactionIsolation(int)
     */
    public void setTransactionIsolation( int level )
        throws SQLException
    {
        connection.setTransactionIsolation( level );
    }

    /**
     * {@inheritDoc}
     * @see java.sql.Connection#setTypeMap(java.util.Map)
     */
    public void setTypeMap( Map arg0 )
        throws SQLException
    {
        connection.setTypeMap( arg0 );
    }

    /**
     * @return the connection
     */
    protected Connection getTargetConnection()
    {
        return connection;
    }

    /**
     * @param connection target connecion
     * @param monitor monitor for opened connections
     * @return monitored connection
     */
    public static Connection wrapp( Connection connection, Monitor monitor )
    {
        if ( RuntimeDiscovery.isJava14() )
        {
            Object[] args = new Object[] { connection, monitor };
            return (Connection) FactorySupport.silentlyCreate( JDBC3_CONNECTION, args );
        }
        return new Jdbc2MonitoredConnection( connection, monitor );
    }

    /**
     * @param statement traget Statement
     * @return monitored Statement
     */
    private Statement wrapp( Statement statement )
    {
        if ( RuntimeDiscovery.isJava14() )
        {
            Object[] args = new Object[] { statement, this };
            return (Statement) FactorySupport.silentlyCreate( JDBC3_STATEMENT, args );
        }
        return new Jdbc2MonitoredStatement( statement, this );
    }

    /**
     * @param statement traget PreparedStatement
     * @param sql SQL Query
     * @return monitored PreparedStatement
     */
    private PreparedStatement wrapp( PreparedStatement statement, String sql )
    {
        if ( RuntimeDiscovery.isJava14() )
        {
            Object[] args = new Object[] { statement, sql, this };
            return (PreparedStatement) FactorySupport.silentlyCreate( JDBC3_PREPARED_STATEMENT, args );
        }
        return new Jdbc2MonitoredPreparedStatement( statement, sql, this );
    }

    /**
     * @param statement target PreparedStatement
     * @param sql SQL Query
     * @return Monitored CallableStatement
     */
    private CallableStatement wrapp( CallableStatement statement, String sql )
    {
        if ( RuntimeDiscovery.isJava14() )
        {
            Object[] args = new Object[] { statement, sql, this };
            return (CallableStatement) FactorySupport.silentlyCreate( JDBC3_CALLABLE_STATEMENT, args );
        }
        return new Jdbc2MonitoredCallableStatement( statement, sql, this );
    }

    /**
     * @return the domain
     */
    public String getDomain()
    {
        return domain;
    }
}
