/*
 ~ Copyright 2006-2007 Nicolas De Loof.
 ~
 ~ Licensed under the Apache License, Version 2.0 (the "License");
 ~ you may not use this file except in compliance with the License.
 ~ You may obtain a copy of the License at
 ~
 ~      http://www.apache.org/licenses/LICENSE-2.0
 ~
 ~ Unless required by applicable law or agreed to in writing, software
 ~ distributed under the License is distributed on an "AS IS" BASIS,
 ~ WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 ~ See the License for the specific language governing permissions and
 ~ limitations under the License.
 */
package info.jmonit.support.jdbc;

import info.jmonit.Monitor;
import info.jmonit.Monitoring;

import java.io.PrintWriter;
import java.sql.Connection;
import java.sql.SQLException;

import javax.sql.DataSource;

/**
 * DataSource wrapper that adds monitoring on JDBC connexions.
 * 
 * @author <a href="mailto:ndeloof@sourceforge.net">ndeloof</a>
 */
public class MonitoredDataSource
    implements DataSource
{
    /** delegate DataSource */
    private DataSource dataSource;

    /** dataSource name */
    private String dataSourceName = "dataSource";
    
    /** Monitor for datasource opened connections */
    private Monitor monitor;

    /**
     * @param dataSource the dataSource to set
     */
    protected void setDataSource( DataSource dataSource )
    {
        this.dataSource = dataSource;
    }

    /**
     * @param dataSourceName the dataSourceName to set
     */
    protected void setDataSourceName( String dataSourceName )
    {
        this.dataSourceName = dataSourceName;
    }

    /**
     * @param monitor the monitor to set
     */
    protected void setMonitor( Monitor monitor )
    {
        this.monitor = monitor;
    }
    
    /**
     * Init
     * @return DataSource ready for use
     */
    public DataSource init()
    {
        if ( monitor == null )
        {
            monitor = Monitoring.getMonitor( "jdbc.connections", dataSourceName );
        }
        return this;
    }

    /**
     * {@inheritDoc}
     * 
     * @see javax.sql.DataSource#getConnection()
     */
    public Connection getConnection()
        throws SQLException
    {
        Connection connection = dataSource.getConnection();
        return MonitoredConnection.wrapp( connection, monitor );
    }

    /**
     * {@inheritDoc}
     * 
     * @see javax.sql.DataSource#getConnection(java.lang.String, java.lang.String)
     */
    public Connection getConnection( String username, String password )
        throws SQLException
    {
        Connection connection = dataSource.getConnection( username, password );
        return MonitoredConnection.wrapp( connection, monitor );
    }

    /**
     * {@inheritDoc}
     * 
     * @see javax.sql.DataSource#getLoginTimeout()
     */
    public int getLoginTimeout()
        throws SQLException
    {
        return dataSource.getLoginTimeout();
    }

    /**
     * {@inheritDoc}
     * 
     * @see javax.sql.DataSource#getLogWriter()
     */
    public PrintWriter getLogWriter()
        throws SQLException
    {
        return dataSource.getLogWriter();
    }

    /**
     * {@inheritDoc}
     * 
     * @see javax.sql.DataSource#setLoginTimeout(int)
     */
    public void setLoginTimeout( int seconds )
        throws SQLException
    {
        dataSource.setLoginTimeout( seconds );
    }

    /**
     * {@inheritDoc}
     * 
     * @see javax.sql.DataSource#setLogWriter(java.io.PrintWriter)
     */
    public void setLogWriter( PrintWriter out )
        throws SQLException
    {
        dataSource.setLogWriter( out );
    }
}
