/*
 ~ Copyright 2006-2007 Nicolas De Loof.
 ~
 ~ Licensed under the Apache License, Version 2.0 (the "License");
 ~ you may not use this file except in compliance with the License.
 ~ You may obtain a copy of the License at
 ~
 ~      http://www.apache.org/licenses/LICENSE-2.0
 ~
 ~ Unless required by applicable law or agreed to in writing, software
 ~ distributed under the License is distributed on an "AS IS" BASIS,
 ~ WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 ~ See the License for the specific language governing permissions and
 ~ limitations under the License.
 */
package info.jmonit.support.jdbc;

import info.jmonit.Monitoring;
import info.jmonit.Probe;

import java.io.InputStream;
import java.io.Reader;
import java.math.BigDecimal;
import java.sql.Array;
import java.sql.Blob;
import java.sql.CallableStatement;
import java.sql.Clob;
import java.sql.Connection;
import java.sql.Date;
import java.sql.PreparedStatement;
import java.sql.Ref;
import java.sql.ResultSet;
import java.sql.ResultSetMetaData;
import java.sql.SQLException;
import java.sql.SQLWarning;
import java.sql.Statement;
import java.sql.Time;
import java.sql.Timestamp;
import java.util.Calendar;
import java.util.Map;
import java.util.StringTokenizer;

/**
 * Wrapped to monitor JDBC Statement
 *
 * @author <a href="mailto:ndeloof@sourceforge.net">ndeloof</a>
 */
public abstract class MonitoredStatement
    implements Statement
{
    /** Monitored connection */
    private MonitoredConnection connection;

    /** delegate statement */
    private Statement statement;

    /** delegate preparedStatement */
    private PreparedStatement preparedStatement;

    /** delegate callableStatement */
    private CallableStatement callableStatement;

    /** The SQL query executed by this statement */
    private String query;
    
    /** jMonit domain for JDBC monitoring */
    private String domain;
    
    /**
     * @param statement target statement
     * @param connection monitored connection
     */
    public MonitoredStatement( Statement statement, MonitoredConnection connection )
    {
        super();
        this.statement = statement;
        this.connection = connection;
        this.domain = connection.getDomain();
    }

    /**
     * @param statement target statement
     * @param query SQL query
     * @param connection monitored connection
     */
    public MonitoredStatement( PreparedStatement statement, String query, MonitoredConnection connection )
    {
        this( statement, connection );
        this.statement = statement;
        this.preparedStatement = statement;
        this.query = query;
    }

    /**
     * @param statement target statement
     * @param query SQL query
     * @param connection monitored connection
     */
    public MonitoredStatement( CallableStatement statement, String query, MonitoredConnection connection )
    {
        this( statement, connection );
        this.statement = statement;
        this.preparedStatement = statement;
        this.callableStatement = statement;
        this.query = query;
    }

    /**
     * Convert a SQL query to a "generic" query by removing any hard coded value.
     * This makes the SQL look like a PreparedStatement.
     *
     * @param sql SQL query
     * @return generalized SQL query
     */
    protected String generalizeSql( String sql )
    {
        StringBuffer stb = new StringBuffer();
        StringTokenizer tokenizer = new StringTokenizer( sql, " \t\r\n,)=", true );
        boolean ws = false;
        while ( tokenizer.hasMoreTokens() )
        {
            String token = tokenizer.nextToken();
            if ( token.length() == 1 && Character.isWhitespace( token.charAt( 0 ) ) )
            {
                if ( !ws )
                {
                    stb.append( " " );
                }
                ws = true;
                continue;
            }
            ws = false;
            char first = token.charAt( 0 );
            char last = token.charAt( token.length() - 1 );
            if ( first == last && ( first == '\'' || first == '"' ) )
            {
                // Quoted value
                token = "?";
            }
            else
            {
                boolean numeric = true;
                char[] chars = token.toCharArray();
                boolean dot = false;
                for ( int i = 0; i < chars.length; i++ )
                {
                    if ( !Character.isDigit( chars[i] ) && !dot && chars[i] != '.' )
                    {
                        numeric = false;
                        break;
                    }
                    if ( chars[i] == '.' )
                    {
                        dot = true;
                    }
                }
                if ( numeric )
                {
                    // Hard coded Numeric
                    token = "?";
                }
            }
            stb.append( token );
        }
        return stb.toString().toLowerCase();
    }

    /**
     * @param sqle catched exception
     * @return SQLException exception to throw
     */
    protected SQLException handleException( SQLException sqle )
    {
        Monitoring.add( domain + ".SQLExceptions", String.valueOf( sqle.getErrorCode() ), 1 );
        return sqle;
    }

    /**
     * {@inheritDoc}
     *
     * @see java.sql.Statement#addBatch(java.lang.String)
     */
    public void addBatch( String sql )
        throws SQLException
    {
        statement.addBatch( sql );
    }

    /**
     * {@inheritDoc}
     *
     * @see java.sql.Statement#cancel()
     */
    public void cancel()
        throws SQLException
    {
        statement.cancel();
    }

    /**
     * {@inheritDoc}
     *
     * @see java.sql.Statement#clearBatch()
     */
    public void clearBatch()
        throws SQLException
    {
        statement.clearBatch();
    }

    /**
     * {@inheritDoc}
     *
     * @see java.sql.Statement#clearWarnings()
     */
    public void clearWarnings()
        throws SQLException
    {
        statement.clearWarnings();
    }

    /**
     * {@inheritDoc}
     *
     * @see java.sql.Statement#close()
     */
    public void close()
        throws SQLException
    {
        statement.close();
    }

    /**
     * {@inheritDoc}
     *
     * @see java.sql.Statement#execute(java.lang.String)
     */
    public boolean execute( String sql )
        throws SQLException
    {
        Probe probe = Monitoring.start( domain + ".statements", generalizeSql( sql ) );
        try
        {
            return statement.execute( sql );
        }
        catch ( SQLException sqle )
        {
            throw handleException( sqle );
        }
        finally
        {
            probe.stop();
        }
    }

    /**
     * {@inheritDoc}
     *
     * @see java.sql.Statement#executeBatch()
     */
    public int[] executeBatch()
        throws SQLException
    {
        Probe probe = Monitoring.start( domain + ".statements", "batch" );
        try
        {
            return statement.executeBatch();
        }
        catch ( SQLException sqle )
        {
            throw handleException( sqle );
        }
        finally
        {
            probe.stop();
        }
    }

    /**
     * {@inheritDoc}
     *
     * @see java.sql.Statement#executeQuery(java.lang.String)
     */
    public ResultSet executeQuery( String sql )
        throws SQLException
    {
        Probe probe = Monitoring.start( domain + ".statements", generalizeSql( sql ) );
        try
        {
            return statement.executeQuery( sql );
        }
        catch ( SQLException sqle )
        {
            throw handleException( sqle );
        }
        finally
        {
            probe.stop();
        }
    }

    /**
     * {@inheritDoc}
     *
     * @see java.sql.Statement#executeUpdate(java.lang.String)
     */
    public int executeUpdate( String sql )
        throws SQLException
    {
        Probe probe = Monitoring.start( domain + ".statements", generalizeSql( sql ) );
        try
        {
            return statement.executeUpdate( sql );
        }
        catch ( SQLException sqle )
        {
            throw handleException( sqle );
        }
        finally
        {
            probe.stop();
        }
    }

    /**
     * {@inheritDoc}
     *
     * @see java.sql.Statement#getConnection()
     */
    public Connection getConnection()
        throws SQLException
    {
        return connection;
    }

    /**
     * {@inheritDoc}
     *
     * @see java.sql.Statement#getFetchDirection()
     */
    public int getFetchDirection()
        throws SQLException
    {
        return statement.getFetchDirection();
    }

    /**
     * {@inheritDoc}
     *
     * @see java.sql.Statement#getFetchSize()
     */
    public int getFetchSize()
        throws SQLException
    {
        return statement.getFetchSize();
    }

    /**
     * {@inheritDoc}
     *
     * @see java.sql.Statement#getMaxFieldSize()
     */
    public int getMaxFieldSize()
        throws SQLException
    {
        return statement.getMaxFieldSize();
    }

    /**
     * {@inheritDoc}
     *
     * @see java.sql.Statement#getMaxRows()
     */
    public int getMaxRows()
        throws SQLException
    {
        return statement.getMaxRows();
    }

    /**
     * {@inheritDoc}
     *
     * @see java.sql.Statement#getMoreResults()
     */
    public boolean getMoreResults()
        throws SQLException
    {
        return statement.getMoreResults();
    }

    /**
     * {@inheritDoc}
     *
     * @see java.sql.Statement#getQueryTimeout()
     */
    public int getQueryTimeout()
        throws SQLException
    {
        return statement.getQueryTimeout();
    }

    /**
     * {@inheritDoc}
     *
     * @see java.sql.Statement#getResultSet()
     */
    public ResultSet getResultSet()
        throws SQLException
    {
        return statement.getResultSet();
    }

    /**
     * {@inheritDoc}
     *
     * @see java.sql.Statement#getResultSetConcurrency()
     */
    public int getResultSetConcurrency()
        throws SQLException
    {
        return statement.getResultSetConcurrency();
    }

    /**
     * {@inheritDoc}
     *
     * @see java.sql.Statement#getResultSetType()
     */
    public int getResultSetType()
        throws SQLException
    {
        return statement.getResultSetType();
    }

    /**
     * {@inheritDoc}
     *
     * @see java.sql.Statement#getUpdateCount()
     */
    public int getUpdateCount()
        throws SQLException
    {
        return statement.getUpdateCount();
    }

    /**
     * {@inheritDoc}
     *
     * @see java.sql.Statement#getWarnings()
     */
    public SQLWarning getWarnings()
        throws SQLException
    {
        return statement.getWarnings();
    }

    /**
     * {@inheritDoc}
     *
     * @see java.sql.Statement#setCursorName(java.lang.String)
     */
    public void setCursorName( String name )
        throws SQLException
    {
        statement.setCursorName( name );
    }

    /**
     * {@inheritDoc}
     *
     * @see java.sql.Statement#setEscapeProcessing(boolean)
     */
    public void setEscapeProcessing( boolean enable )
        throws SQLException
    {
        statement.setEscapeProcessing( enable );
    }

    /**
     * {@inheritDoc}
     *
     * @see java.sql.Statement#setFetchDirection(int)
     */
    public void setFetchDirection( int direction )
        throws SQLException
    {
        statement.setFetchDirection( direction );
    }

    /**
     * {@inheritDoc}
     *
     * @see java.sql.Statement#setFetchSize(int)
     */
    public void setFetchSize( int rows )
        throws SQLException
    {
        statement.setFetchSize( rows );
    }

    /**
     * {@inheritDoc}
     *
     * @see java.sql.Statement#setMaxFieldSize(int)
     */
    public void setMaxFieldSize( int max )
        throws SQLException
    {
        statement.setMaxFieldSize( max );
    }

    /**
     * {@inheritDoc}
     *
     * @see java.sql.Statement#setMaxRows(int)
     */
    public void setMaxRows( int max )
        throws SQLException
    {
        statement.setMaxRows( max );
    }

    /**
     * {@inheritDoc}
     *
     * @see java.sql.Statement#setQueryTimeout(int)
     */
    public void setQueryTimeout( int seconds )
        throws SQLException
    {
        statement.setQueryTimeout( seconds );
    }

    /**
     * {@inheritDoc}
     *
     * @see java.sql.PreparedStatement#addBatch()
     */
    public void addBatch()
        throws SQLException
    {
        preparedStatement.addBatch();
    }

    /**
     * {@inheritDoc}
     *
     * @see java.sql.PreparedStatement#clearParameters()
     */
    public void clearParameters()
        throws SQLException
    {
        preparedStatement.clearParameters();
    }

    /**
     * {@inheritDoc}
     *
     * @see java.sql.PreparedStatement#execute()
     */
    public boolean execute()
        throws SQLException
    {
        Probe probe = Monitoring.start( domain + ".preparedStatements", query );
        try
        {
            return preparedStatement.execute();
        }
        catch ( SQLException sqle )
        {
            throw handleException( sqle );
        }
        finally
        {
            probe.stop();
        }
    }

    /**
     * {@inheritDoc}
     *
     * @see java.sql.PreparedStatement#executeQuery()
     */
    public ResultSet executeQuery()
        throws SQLException
    {
        Probe probe = Monitoring.start( domain + ".preparedStatements", query );
        try
        {
            return preparedStatement.executeQuery();
        }
        catch ( SQLException sqle )
        {
            throw handleException( sqle );
        }
        finally
        {
            probe.stop();
        }
    }

    /**
     * {@inheritDoc}
     *
     * @see java.sql.PreparedStatement#executeUpdate()
     */
    public int executeUpdate()
        throws SQLException
    {
        Probe probe = Monitoring.start( domain + ".preparedStatements", query );
        try
        {
            return preparedStatement.executeUpdate();
        }
        catch ( SQLException sqle )
        {
            throw handleException( sqle );
        }
        finally
        {
            probe.stop();
        }
    }

    /**
     * {@inheritDoc}
     *
     * @see java.sql.PreparedStatement#getMetaData()
     */
    public ResultSetMetaData getMetaData()
        throws SQLException
    {
        return preparedStatement.getMetaData();
    }

    /**
     * {@inheritDoc}
     *
     * @see java.sql.PreparedStatement#setArray(int, java.sql.Array)
     */
    public void setArray( int i, Array x )
        throws SQLException
    {
        preparedStatement.setArray( i, x );
    }

    /**
     * {@inheritDoc}
     *
     * @see java.sql.PreparedStatement#setAsciiStream(int, java.io.InputStream, int)
     */
    public void setAsciiStream( int parameterIndex, InputStream x, int length )
        throws SQLException
    {
        preparedStatement.setAsciiStream( parameterIndex, x, length );
    }

    /**
     * {@inheritDoc}
     *
     * @see java.sql.PreparedStatement#setBigDecimal(int, java.math.BigDecimal)
     */
    public void setBigDecimal( int parameterIndex, BigDecimal x )
        throws SQLException
    {
        preparedStatement.setBigDecimal( parameterIndex, x );
    }

    /**
     * {@inheritDoc}
     *
     * @see java.sql.PreparedStatement#setBinaryStream(int, java.io.InputStream, int)
     */
    public void setBinaryStream( int parameterIndex, InputStream x, int length )
        throws SQLException
    {
        preparedStatement.setBinaryStream( parameterIndex, x, length );
    }

    /**
     * {@inheritDoc}
     *
     * @see java.sql.PreparedStatement#setBlob(int, java.sql.Blob)
     */
    public void setBlob( int i, Blob x )
        throws SQLException
    {
        preparedStatement.setBlob( i, x );
    }

    /**
     * {@inheritDoc}
     *
     * @see java.sql.PreparedStatement#setBoolean(int, boolean)
     */
    public void setBoolean( int parameterIndex, boolean x )
        throws SQLException
    {
        preparedStatement.setBoolean( parameterIndex, x );
    }

    /**
     * {@inheritDoc}
     *
     * @see java.sql.PreparedStatement#setByte(int, byte)
     */
    public void setByte( int parameterIndex, byte x )
        throws SQLException
    {
        preparedStatement.setByte( parameterIndex, x );
    }

    /**
     * {@inheritDoc}
     *
     * @see java.sql.PreparedStatement#setBytes(int, byte[])
     */
    public void setBytes( int parameterIndex, byte[] x )
        throws SQLException
    {
        preparedStatement.setBytes( parameterIndex, x );
    }

    /**
     * {@inheritDoc}
     *
     * @see java.sql.PreparedStatement#setCharacterStream(int, java.io.Reader, int)
     */
    public void setCharacterStream( int parameterIndex, Reader reader, int length )
        throws SQLException
    {
        preparedStatement.setCharacterStream( parameterIndex, reader, length );
    }

    /**
     * {@inheritDoc}
     *
     * @see java.sql.PreparedStatement#setClob(int, java.sql.Clob)
     */
    public void setClob( int i, Clob x )
        throws SQLException
    {
        preparedStatement.setClob( i, x );
    }

    /**
     * {@inheritDoc}
     *
     * @see java.sql.PreparedStatement#setDate(int, java.sql.Date, java.util.Calendar)
     */
    public void setDate( int parameterIndex, Date x, Calendar cal )
        throws SQLException
    {
        preparedStatement.setDate( parameterIndex, x, cal );
    }

    /**
     * {@inheritDoc}
     *
     * @see java.sql.PreparedStatement#setDate(int, java.sql.Date)
     */
    public void setDate( int parameterIndex, Date x )
        throws SQLException
    {
        preparedStatement.setDate( parameterIndex, x );
    }

    /**
     * {@inheritDoc}
     *
     * @see java.sql.PreparedStatement#setDouble(int, double)
     */
    public void setDouble( int parameterIndex, double x )
        throws SQLException
    {
        preparedStatement.setDouble( parameterIndex, x );
    }

    /**
     * {@inheritDoc}
     *
     * @see java.sql.PreparedStatement#setFloat(int, float)
     */
    public void setFloat( int parameterIndex, float x )
        throws SQLException
    {
        preparedStatement.setFloat( parameterIndex, x );
    }

    /**
     * {@inheritDoc}
     *
     * @see java.sql.PreparedStatement#setInt(int, int)
     */
    public void setInt( int parameterIndex, int x )
        throws SQLException
    {
        preparedStatement.setInt( parameterIndex, x );
    }

    /**
     * {@inheritDoc}
     *
     * @see java.sql.PreparedStatement#setLong(int, long)
     */
    public void setLong( int parameterIndex, long x )
        throws SQLException
    {
        preparedStatement.setLong( parameterIndex, x );
    }

    /**
     * {@inheritDoc}
     *
     * @see java.sql.PreparedStatement#setNull(int, int, java.lang.String)
     */
    public void setNull( int paramIndex, int sqlType, String typeName )
        throws SQLException
    {
        preparedStatement.setNull( paramIndex, sqlType, typeName );
    }

    /**
     * {@inheritDoc}
     *
     * @see java.sql.PreparedStatement#setNull(int, int)
     */
    public void setNull( int parameterIndex, int sqlType )
        throws SQLException
    {
        preparedStatement.setNull( parameterIndex, sqlType );
    }

    /**
     * {@inheritDoc}
     *
     * @see java.sql.PreparedStatement#setObject(int, java.lang.Object, int, int)
     */
    public void setObject( int parameterIndex, Object x, int targetSqlType, int scale )
        throws SQLException
    {
        preparedStatement.setObject( parameterIndex, x, targetSqlType, scale );
    }

    /**
     * {@inheritDoc}
     *
     * @see java.sql.PreparedStatement#setObject(int, java.lang.Object, int)
     */
    public void setObject( int parameterIndex, Object x, int targetSqlType )
        throws SQLException
    {
        preparedStatement.setObject( parameterIndex, x, targetSqlType );
    }

    /**
     * {@inheritDoc}
     *
     * @see java.sql.PreparedStatement#setObject(int, java.lang.Object)
     */
    public void setObject( int parameterIndex, Object x )
        throws SQLException
    {
        preparedStatement.setObject( parameterIndex, x );
    }

    /**
     * {@inheritDoc}
     *
     * @see java.sql.PreparedStatement#setRef(int, java.sql.Ref)
     */
    public void setRef( int i, Ref x )
        throws SQLException
    {
        preparedStatement.setRef( i, x );
    }

    /**
     * {@inheritDoc}
     *
     * @see java.sql.PreparedStatement#setShort(int, short)
     */
    public void setShort( int parameterIndex, short x )
        throws SQLException
    {
        preparedStatement.setShort( parameterIndex, x );
    }

    /**
     * {@inheritDoc}
     *
     * @see java.sql.PreparedStatement#setString(int, java.lang.String)
     */
    public void setString( int parameterIndex, String x )
        throws SQLException
    {
        preparedStatement.setString( parameterIndex, x );
    }

    /**
     * {@inheritDoc}
     *
     * @see java.sql.PreparedStatement#setTime(int, java.sql.Time, java.util.Calendar)
     */
    public void setTime( int parameterIndex, Time x, Calendar cal )
        throws SQLException
    {
        preparedStatement.setTime( parameterIndex, x, cal );
    }

    /**
     * {@inheritDoc}
     *
     * @see java.sql.PreparedStatement#setTime(int, java.sql.Time)
     */
    public void setTime( int parameterIndex, Time x )
        throws SQLException
    {
        preparedStatement.setTime( parameterIndex, x );
    }

    /**
     * {@inheritDoc}
     *
     * @see java.sql.PreparedStatement#setTimestamp(int, java.sql.Timestamp, java.util.Calendar)
     */
    public void setTimestamp( int parameterIndex, Timestamp x, Calendar cal )
        throws SQLException
    {
        preparedStatement.setTimestamp( parameterIndex, x, cal );
    }

    /**
     * {@inheritDoc}
     *
     * @see java.sql.PreparedStatement#setTimestamp(int, java.sql.Timestamp)
     */
    public void setTimestamp( int parameterIndex, Timestamp x )
        throws SQLException
    {
        preparedStatement.setTimestamp( parameterIndex, x );
    }

    /**
     * {@inheritDoc}
     *
     * @see java.sql.PreparedStatement#setUnicodeStream(int, java.io.InputStream, int)
     */
    public void setUnicodeStream( int parameterIndex, InputStream x, int length )
        throws SQLException
    {
        preparedStatement.setUnicodeStream( parameterIndex, x, length );
    }

    /**
     * {@inheritDoc}
     * @see java.sql.CallableStatement#getArray(int)
     */
    public Array getArray( int i )
        throws SQLException
    {
        return callableStatement.getArray( i );
    }

    /**
     * {@inheritDoc}
     * @see java.sql.CallableStatement#getBigDecimal(int, int)
     */
    public BigDecimal getBigDecimal( int parameterIndex, int scale )
        throws SQLException
    {
        return callableStatement.getBigDecimal( parameterIndex, scale );
    }

    /**
     * {@inheritDoc}
     * @see java.sql.CallableStatement#getBigDecimal(int)
     */
    public BigDecimal getBigDecimal( int parameterIndex )
        throws SQLException
    {
        return callableStatement.getBigDecimal( parameterIndex );
    }

    /**
     * {@inheritDoc}
     * @see java.sql.CallableStatement#getBlob(int)
     */
    public Blob getBlob( int i )
        throws SQLException
    {
        return callableStatement.getBlob( i );
    }

    /**
     * {@inheritDoc}
     * @see java.sql.CallableStatement#getBoolean(int)
     */
    public boolean getBoolean( int parameterIndex )
        throws SQLException
    {
        return callableStatement.getBoolean( parameterIndex );
    }

    /**
     * {@inheritDoc}
     * @see java.sql.CallableStatement#getByte(int)
     */
    public byte getByte( int parameterIndex )
        throws SQLException
    {
        return callableStatement.getByte( parameterIndex );
    }

    /**
     * {@inheritDoc}
     * @see java.sql.CallableStatement#getBytes(int)
     */
    public byte[] getBytes( int parameterIndex )
        throws SQLException
    {
        return callableStatement.getBytes( parameterIndex );
    }

    /**
     * {@inheritDoc}
     * @see java.sql.CallableStatement#getClob(int)
     */
    public Clob getClob( int i )
        throws SQLException
    {
        return callableStatement.getClob( i );
    }

    /**
     * {@inheritDoc}
     * @see java.sql.CallableStatement#getDate(int, java.util.Calendar)
     */
    public Date getDate( int parameterIndex, Calendar cal )
        throws SQLException
    {
        return callableStatement.getDate( parameterIndex, cal );
    }

    /**
     * {@inheritDoc}
     * @see java.sql.CallableStatement#getDate(int)
     */
    public Date getDate( int parameterIndex )
        throws SQLException
    {
        return callableStatement.getDate( parameterIndex );
    }

    /**
     * {@inheritDoc}
     * @see java.sql.CallableStatement#getDouble(int)
     */
    public double getDouble( int parameterIndex )
        throws SQLException
    {
        return callableStatement.getDouble( parameterIndex );
    }

    /**
     * {@inheritDoc}
     * @see java.sql.CallableStatement#getFloat(int)
     */
    public float getFloat( int parameterIndex )
        throws SQLException
    {
        return callableStatement.getFloat( parameterIndex );
    }

    /**
     * {@inheritDoc}
     * @see java.sql.CallableStatement#getInt(int)
     */
    public int getInt( int parameterIndex )
        throws SQLException
    {
        return callableStatement.getInt( parameterIndex );
    }

    /**
     * {@inheritDoc}
     * @see java.sql.CallableStatement#getLong(int)
     */
    public long getLong( int parameterIndex )
        throws SQLException
    {
        return callableStatement.getLong( parameterIndex );
    }

    /**
     * {@inheritDoc}
     * @see java.sql.CallableStatement#getObject(int, java.util.Map)
     */
    public Object getObject( int arg0, Map arg1 )
        throws SQLException
    {
        return callableStatement.getObject( arg0, arg1 );
    }

    /**
     * {@inheritDoc}
     * @see java.sql.CallableStatement#getObject(int)
     */
    public Object getObject( int parameterIndex )
        throws SQLException
    {
        return callableStatement.getObject( parameterIndex );
    }

    /**
     * {@inheritDoc}
     * @see java.sql.CallableStatement#getRef(int)
     */
    public Ref getRef( int i )
        throws SQLException
    {
        return callableStatement.getRef( i );
    }

    /**
     * {@inheritDoc}
     * @see java.sql.CallableStatement#getShort(int)
     */
    public short getShort( int parameterIndex )
        throws SQLException
    {
        return callableStatement.getShort( parameterIndex );
    }

    /**
     * {@inheritDoc}
     * @see java.sql.CallableStatement#getString(int)
     */
    public String getString( int parameterIndex )
        throws SQLException
    {
        return callableStatement.getString( parameterIndex );
    }

    /**
     * {@inheritDoc}
     * @see java.sql.CallableStatement#getTime(int, java.util.Calendar)
     */
    public Time getTime( int parameterIndex, Calendar cal )
        throws SQLException
    {
        return callableStatement.getTime( parameterIndex, cal );
    }

    /**
     * {@inheritDoc}
     * @see java.sql.CallableStatement#getTime(int)
     */
    public Time getTime( int parameterIndex )
        throws SQLException
    {
        return callableStatement.getTime( parameterIndex );
    }

    /**
     * {@inheritDoc}
     * @see java.sql.CallableStatement#getTimestamp(int, java.util.Calendar)
     */
    public Timestamp getTimestamp( int parameterIndex, Calendar cal )
        throws SQLException
    {
        return callableStatement.getTimestamp( parameterIndex, cal );
    }

    /**
     * {@inheritDoc}
     * @see java.sql.CallableStatement#getTimestamp(int)
     */
    public Timestamp getTimestamp( int parameterIndex )
        throws SQLException
    {
        return callableStatement.getTimestamp( parameterIndex );
    }

    /**
     * {@inheritDoc}
     * @see java.sql.CallableStatement#registerOutParameter(int, int, int)
     */
    public void registerOutParameter( int parameterIndex, int sqlType, int scale )
        throws SQLException
    {
        callableStatement.registerOutParameter( parameterIndex, sqlType, scale );
    }

    /**
     * {@inheritDoc}
     * @see java.sql.CallableStatement#registerOutParameter(int, int, java.lang.String)
     */
    public void registerOutParameter( int paramIndex, int sqlType, String typeName )
        throws SQLException
    {
        callableStatement.registerOutParameter( paramIndex, sqlType, typeName );
    }

    /**
     * {@inheritDoc}
     * @see java.sql.CallableStatement#registerOutParameter(int, int)
     */
    public void registerOutParameter( int parameterIndex, int sqlType )
        throws SQLException
    {
        callableStatement.registerOutParameter( parameterIndex, sqlType );
    }

    /**
     * {@inheritDoc}
     * @see java.sql.CallableStatement#wasNull()
     */
    public boolean wasNull()
        throws SQLException
    {
        return callableStatement.wasNull();
    }
}
