/*
 ~ Copyright 2006-2007 Nicolas De Loof.
 ~
 ~ Licensed under the Apache License, Version 2.0 (the "License");
 ~ you may not use this file except in compliance with the License.
 ~ You may obtain a copy of the License at
 ~
 ~      http://www.apache.org/licenses/LICENSE-2.0
 ~
 ~ Unless required by applicable law or agreed to in writing, software
 ~ distributed under the License is distributed on an "AS IS" BASIS,
 ~ WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 ~ See the License for the specific language governing permissions and
 ~ limitations under the License.
 */
package info.jmonit.support.soap;

import info.jmonit.Monitoring;
import info.jmonit.Probe;
import info.jmonit.logger.Logger;
import info.jmonit.logger.LoggerFactory;

import javax.xml.rpc.Call;
import javax.xml.rpc.handler.soap.SOAPMessageContext;

import org.apache.axis.AxisFault;
import org.apache.axis.Message;
import org.apache.axis.MessageContext;
import org.apache.axis.handlers.BasicHandler;

/**
 * Axis handler to monitor Time for SAOP invocation. Three jMonit domains are
 * used for Axis SOAP monitoring :
 * <ul>
 * <li><b>axis</b> for timing.</li>
 * <li><b>axis.sent</b> for bytes sent.</li>
 * <li><b>axis.received</b> for bytes received.</li>
 * </ul>
 * <p>
 * You have to configure your Axis client by including in the client-config.wsdd
 * file (Axis client configuration file):
 *
 * <pre>
 *      &lt;?xml version=&quot;1.0&quot; encoding=&quot;UTF-8&quot;?&gt;
 *      &lt;deployment xmlns=&quot;http://xml.apache.org/axis/wsdd/&quot;
 *      xmlns:java=&quot;http://xml.apache.org/axis/wsdd/providers/java&quot;&gt;
 *
 *        &lt;handler name=&quot;jmonit&quot; type=&quot;java:info.jmonit.support.soap.AxisMonitoringHandler&quot;&gt;
 *          &lt;parameter name=&quot;domain&quot; value=&quot;soap.axis&quot;/&gt;
 *          &lt;parameter name=&quot;requestDomain&quot; value=&quot;soap.axis.request&quot;/&gt;
 *          &lt;parameter name=&quot;responseDomain&quot; value=&quot;soap.axis.response&quot;/&gt;
 *        &lt;/handler&gt;
 *
 *        &lt;transport name=&quot;http&quot;
 *        pivot=&quot;java:org.apache.axis.transport.http.HTTPSender&quot;&gt;
 *          &lt;requestFlow&gt;
 *            &lt;handler type=&quot;jmonit&quot;/&gt;
 *          &lt;/requestFlow&gt;
 *          &lt;responseFlow&gt;
 *            &lt;handler type=&quot;jmonit&quot;/&gt;
 *          &lt;/responseFlow&gt;
 *        &lt;/transport&gt;
 *
 *      &lt;/deployment&gt;
 * </pre>
 *
 * Handler parameter can be used to override default domains. This can be
 * usefull if you are both using Axis as Web service client and as server-side
 * SOAP stack, where request/response have inverted roles.
 *
 * @author <a href="mailto:ndeloof@sourceforge.net">ndeloof</a>
 */
public class AxisMonitoringHandler
    extends BasicHandler
{
    /** Logger */
    private static Logger logger = LoggerFactory.getLogger( AxisMonitoringHandler.class );

    /**
     * Domain used to monitor time consumed in transport stack, network and
     * remote server
     */
    private String domain;

    /**
     * Domain used to monitor weight (in bytes) of request messages.
     */
    private String requestSubDomain;

    /**
     * Domain used to monitor weight (in bytes) of response messages.
     */
    private String responseSubDomain;

    /**
     * Use axis handler configuration to set monitors domains used.
     *
     * @see org.apache.axis.handlers.BasicHandler#init()
     */
    public void init()
    {
        domain = getOption( "domain", "axis" );
        requestSubDomain = getOption( "requestDomain", "axis.sent" );
        responseSubDomain = getOption( "responseDomain", "axis.received" );
        super.init();
    }

    /**
     * A Probe is put in the Axix messageContext before sending the request SOAP
     * message. It is used when response message is received to get the SOAP
     * timing.
     * <p>
     * {@inheritDoc}
     *
     * @see org.apache.axis.Handler#invoke(org.apache.axis.MessageContext)
     */
    public void invoke( MessageContext msgContext )
        throws AxisFault
    {
        if ( !msgContext.getPastPivot() )
        {
            String soapAction = getSoapAction( msgContext );
            Probe probe = Monitoring.start( domain, soapAction );
            msgContext.setProperty( "jMonitProbe", probe );
        }
        else
        {
            stopMonitoring( msgContext );
        }
    }

    /**
     * {@inheritDoc}
     *
     * @see org.apache.axis.handlers.BasicHandler#onFault(org.apache.axis.MessageContext)
     */
    public void onFault( MessageContext msgContext )
    {
        stopMonitoring( msgContext );
    }

    /**
     * Get handler parameter, and fall back to default value.
     *
     * @param key Axis handler parameter name
     * @param defaultValue defualt value for option
     * @return value for option
     */
    protected String getOption( String key, String defaultValue )
    {
        String value = (String) getOption( key );
        return ( value != null ? value : defaultValue );
    }

    /**
     * Extract the SoapAction from message context, either by getting the
     * Jax-rpc header, or by getting the SOAP message first element.
     * <p>
     * TODO : Does this work for any SOAP message ?
     *
     * @param context Jax-rpc context
     * @return soapAction of the SOAP message
     */
    protected String getSoapAction( SOAPMessageContext context )
    {
        String soapAction;
        try
        {
            soapAction = (String) context.getProperty( Call.SOAPACTION_URI_PROPERTY );
            if ( soapAction == null || soapAction.length() == 0 )
            {
                soapAction = context.getMessage().getSOAPBody().getFirstChild().getNodeName();
            }
        }
        catch ( Exception e )
        {
            logger.warn( "Failed to extract the SOAPAction", e );
            soapAction = "unknown";
        }

        // Experiment show "ns1:" namespace is set on response, and not set
        // on request, so remove any namespace prefix
        int i = soapAction.indexOf( ':' );
        if ( i >= 0 )
        {
            soapAction = soapAction.substring( i + 1 );
        }
        return soapAction;
    }

    /**
     * End monitoring for a SOAP invocation.
     * <p>
     * Stops the running probe and add messages weight to request/response bytes
     * monitors.
     *
     * @param msgContext Axis message context
     */
    protected void stopMonitoring( MessageContext msgContext )
    {
        String soapAction = getSoapAction( msgContext );
        Probe probe = (Probe) msgContext.getProperty( "jMonitProbe" );
        if ( probe != null )
        {
            probe.stop();
        }
        Message request = msgContext.getRequestMessage();
        Message response = msgContext.getResponseMessage();
        try
        {
            Monitoring.getMonitor( requestSubDomain, soapAction ).add( request.getContentLength() );
            Monitoring.getMonitor( responseSubDomain, soapAction ).add( response.getContentLength() );
        }
        catch ( Exception e )
        {
            logger.warn( "Failed to get SOAP message size", e );
        }
    }
}
