/*
 ~ Copyright 2006-2007 Nicolas De Loof.
 ~
 ~ Licensed under the Apache License, Version 2.0 (the "License");
 ~ you may not use this file except in compliance with the License.
 ~ You may obtain a copy of the License at
 ~
 ~      http://www.apache.org/licenses/LICENSE-2.0
 ~
 ~ Unless required by applicable law or agreed to in writing, software
 ~ distributed under the License is distributed on an "AS IS" BASIS,
 ~ WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 ~ See the License for the specific language governing permissions and
 ~ limitations under the License.
 */
package info.jmonit.web;

import info.jmonit.Domain;
import info.jmonit.Monitoring;
import info.jmonit.Probe;
import info.jmonit.Repository;
import info.jmonit.logger.Logger;
import info.jmonit.logger.LoggerFactory;
import info.jmonit.web.ui.Controller;
import info.jmonit.web.ui.SimpleController;

import java.io.IOException;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.Map;
import java.util.Properties;
import java.util.StringTokenizer;

import javax.servlet.Filter;
import javax.servlet.FilterChain;
import javax.servlet.FilterConfig;
import javax.servlet.ServletException;
import javax.servlet.ServletRequest;
import javax.servlet.ServletResponse;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

/**
 * A servlet Filter to be configured in your WEB-INF/web.xml to intercept all
 * incoming request.
 *
 * <pre>
 *         &lt;filter&gt;
 *         &lt;filter-name&gt;jMonit&lt;/filter-name&gt;
 *         &lt;filter-class&gt;info.jmonit.JMonitFilter&lt;/filter-class&gt;
 *         &lt;/filter&gt;
 *         &lt;filter-mapping&gt;
 *         &lt;filter-name&gt;jMonit&lt;/filter-name&gt;
 *         &lt;url-pattern&gt;/*&lt;/url-pattern&gt;
 *         &lt;/filter-mapping&gt;
 * </pre>
 *
 * <p>
 * A monitoring monitor will be created for each application URI. The "/" URI
 * separator is used to create a monitor hierarchy.
 * <p>
 * This filter is also a front controller for the jMonit web UI. By default,
 * request for /jmonit/* will be delegated to the jMonit UI, and basic
 * authentication will be required, with login / password "jmonit"/"jmonit".
 * Authentication can be configured using filter parameters.
 *
 * @author ndeloof
 */
public class JMonitFilter
    implements Filter
{
    /** Logger */
    private static Logger logger = LoggerFactory.getLogger( JMonitFilter.class );

    /** Prefix for web UI URIs */
    private static final String WEB_UI_PREFIX = "/jMonit/";

    /** Configuration param for domain mappings */
    private static final String MAPPINGS = "mappings";

    /** Configuration param default domain name */
    private static final String DEFAULT_DOMAIN = "defaultDomain";

    /** Web UI controller */
    private Controller controller;

    /** Monitoring mappings by requested resource extension */
    private Map mappings = new HashMap();

    /** Repository of monitors */
    private Repository repository;

    /** Default domain for request without matching mapping */
    private String defaultDomain;

    /**
     * {@inheritDoc}
     *
     * @see javax.servlet.Filter#destroy()
     */
    public void destroy()
    {

    }

    /**
     * Delegates to Http based doFilter. {@inheritDoc}
     *
     * @see javax.servlet.Filter#doFilter(javax.servlet.ServletRequest,
     * javax.servlet.ServletResponse, javax.servlet.FilterChain)
     */
    public void doFilter( ServletRequest request, ServletResponse response, FilterChain chain )
        throws IOException, ServletException
    {
        if ( request instanceof HttpServletRequest )
        {
            HttpServletRequest httpRequest = (HttpServletRequest) request;
            HttpServletResponse httpResponse = (HttpServletResponse) response;
            doFilter( httpRequest, httpResponse, chain );
        }
        else
        {
            // Not an HTTP request...
            chain.doFilter( request, response );
        }

    }

    /**
     * Http specific doFilter implementation
     *
     * @param request Http request
     * @param response Http response
     * @param chain request chain
     * @throws IOException IO exception occurs
     * @throws ServletException servlet Exception occurs
     */
    public void doFilter( HttpServletRequest request, HttpServletResponse response, FilterChain chain )
        throws IOException, ServletException
    {
        String uri = request.getRequestURI();
        String context = request.getContextPath();
        uri = uri.substring( context.length() );
        if ( uri.startsWith( WEB_UI_PREFIX ) )
        {
            uri = uri.substring( WEB_UI_PREFIX.length() );
            // jMonit web UI request, delegate to controller
            controller.execute( uri, request, response );
        }
        else
        {
            // Build the Monitoring name
            String name = buildMonitorNameFromUri( uri );
            String domain = getDomainForUri( uri );
            Probe probe = repository.getMonitor( domain, name ).getProbe();
            try
            {
                probe.start();
                chain.doFilter( request, response );
            }
            finally
            {
                probe.stop();
            }
        }
    }

    /**
     * Build the monitor name from request URI.
     * <p>
     * Override this method in a custom filter to define application specific
     * monitors based on URIs.
     *
     * @param uri requested URI
     * @return monitor name
     */
    protected String buildMonitorNameFromUri( String uri )
    {
        return uri;
    }

    /**
     * Build the monitor name from request URI.
     * <p>
     * Override this method in a custom filter to define application specific
     * monitors based on URIs.
     *
     * @param uri requested URI
     * @return domain Domain for requested URI
     */
    protected String getDomainForUri( String uri )
    {
        // TODO support Ant patterns
        int i = uri.lastIndexOf( "." );
        if ( i > 0 )
        {
            String domain = (String) mappings.get( uri.substring( i ) );
            if ( domain != null )
            {
                return domain;
            }
        }
        return defaultDomain;
    }

    /**
     * {@inheritDoc}
     *
     * @see javax.servlet.Filter#init(javax.servlet.FilterConfig)
     */
    public void init( FilterConfig filterConfig )
        throws ServletException
    {
        initController( filterConfig );
        initRepository( filterConfig );
        initDefaultDomain( filterConfig );
        initMapping( filterConfig );
        logger.info( "init done." );
    }

    /**
     * @param filterConfig configuration
     */
    protected void initMapping( FilterConfig filterConfig )
    {
        String maps = filterConfig.getInitParameter( MAPPINGS );
        if ( maps == null )
        {
            // Default mappings
            mappings.put( ".do", "web.struts" );
            mappings.put( ".js", "web.scripts" );
            mappings.put( ".css", "web.resources" );
            mappings.put( ".gif", "web.resources" );
            mappings.put( ".png", "web.resources" );
            mappings.put( ".jpg", "web.resources" );
        }
        else
        {
            StringTokenizer tokenizer = new StringTokenizer( maps );
            while ( tokenizer.hasMoreTokens() )
            {
                String token = tokenizer.nextToken();
                int i = token.indexOf( '=' );
                if ( i < 0 )
                {
                    logger.warn( "Invalid mapping String " + token );
                    continue;
                }
                String domain = token.substring( 0, i ).trim();
                mappings.put( token.substring( i + 1 ).trim(), domain );
            }
        }
    }

    /**
     * @param filterConfig configuration
     */
    protected void initDefaultDomain( FilterConfig filterConfig )
    {
        if ( defaultDomain == null )
        {
            defaultDomain = filterConfig.getInitParameter( DEFAULT_DOMAIN );
            if ( defaultDomain == null )
            {
                defaultDomain = "web";
            }
        }
    }

    /**
     * @param filterConfig configuration
     */
    protected void initRepository( FilterConfig filterConfig )
    {
        if ( repository == null )
        {
            repository = Monitoring.getRepository();
        }
    }

    /**
     * @param filterConfig configuration
     */
    protected void initController( FilterConfig filterConfig )
    {
        Enumeration e = filterConfig.getInitParameterNames();
        Properties properties = new Properties();
        while ( e.hasMoreElements() )
        {
            String name = (String) e.nextElement();
            String value = filterConfig.getInitParameter( name );
            properties.setProperty( name, value );
        }
        // TODO make controller instance configurable
        controller = new SimpleController();
        controller.init( properties );
    }

    /**
     * @param controller The controller to set
     */
    public void setController( Controller controller )
    {
        this.controller = controller;
    }

    /**
     * @param mappings The mappings to set
     */
    public void setMappings( Map mappings )
    {
        this.mappings = mappings;
    }

    /**
     * @param domain the defaultDomain to set
     */
    public void setDefaultDomain( String domain )
    {
        this.defaultDomain = domain;
    }

    /**
     * @param repository the repository to set
     */
    public void setRepository( Repository repository )
    {
        this.repository = repository;
    }

}
