/*
 * ~ Copyright 2006-2007 Nicolas De Loof. ~ ~ Licensed under the Apache License,
 * Version 2.0 (the "License"); ~ you may not use this file except in compliance
 * with the License. ~ You may obtain a copy of the License at ~ ~
 * http://www.apache.org/licenses/LICENSE-2.0 ~ ~ Unless required by applicable
 * law or agreed to in writing, software ~ distributed under the License is
 * distributed on an "AS IS" BASIS, ~ WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied. ~ See the License for the specific language
 * governing permissions and ~ limitations under the License.
 */
package info.jmonit.web.components;

import info.jmonit.Agregator;
import info.jmonit.Domain;
import info.jmonit.Monitor;
import info.jmonit.Monitoring;
import info.jmonit.Repository;
import info.jmonit.visitor.MonitorSnapshot;
import info.jmonit.visitor.Visitor;

import java.io.PrintWriter;
import java.net.URLEncoder;
import java.util.Date;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

/**
 * Web UI DashBoard. s
 *
 * @author <a href="mailto:ndeloof@sourceforge.net">ndeloof</a>
 *
 */
public class DashBoard extends AbstractWebComponent implements Visitor
{
    /** output writer */
    private PrintWriter writer;

    /** rendering state value for repository */
    private static final int REPOSITORY = 0;

    /** rendering state value for domains & sub-domains */
    private static final int DOMAIN = 1;

    /** rendering state value for monitors */
    private static final int MONITOR = 2;

    /** rendering state */
    private int state = REPOSITORY;

    /**
     * ${inheritDoc}
     *
     * @throws Exception
     * @override
     * @see info.jmonit.web.ui.WebComponent#render(javax.servlet.http.HttpServletRequest,
     *      javax.servlet.http.HttpServletResponse)
     */
    public void render( HttpServletRequest request, HttpServletResponse response )
            throws Exception
    {
        this.writer = response.getWriter();
        render( Monitoring.getRepository() );
    }

    public void render( Repository repository )
    {
        head( writer, "DashBoard" );
        repository.accept( this );
        end( writer );
        writer.flush();
    }

    /**
     * {@inheritDoc}
     *
     * @override
     * @see info.jmonit.visitor.MonitorVisitor#visit(info.jmonit.Repository)
     */
    public void visit( Repository repository )
    {
        // Nop
    }

    /**
     * {@inheritDoc}
     *
     * @override
     * @see info.jmonit.visitor.MonitorVisitor#visit(info.jmonit.Domain)
     */
    public void visit( Domain domain )
    {
        if ( state == MONITOR )
        {
            // Close parent-domain monitors table
            writer.println( "</table>" );
        }
        if ( domain.getParent() == null )
        {
            // Root Domain
            writer.println( "<p>" );
            writer.println( dateFormat.format( new Date() ) );
            writer.println( "</p>" );
            writer.println( "<div>" );
        }
        else
        {
            String id = URLEncoder.encode( domain.getName() );
            writer.println( "<p class='domain'>" );
            writer.println( "<img class='action' "
                    + "src='resources/images/less.gif' "
                    + "onclick=\"hide(this, '" + id + "')\"/>" );
            writer.println( domain.getName() );
            writer.println( "</p>" );
            writer.println( "<div id='" + id + "'>" );
        }
        state = DOMAIN;
    }

    /**
     * {@inheritDoc}
     *
     * @see info.jmonit.visitor.MonitorVisitor#visited(info.jmonit.Domain)
     */
    public void visited( Domain domain )
    {
        if ( state == MONITOR )
        {
            // Close monitors table (no sub-domain)
            writer.println( "</table>" );
        }
        // Close domain pannel
        writer.println( "</div>" );
        state = DOMAIN;
    }

    /** flag for odd/even lines style */
    private boolean even;

    /**
     * {@inheritDoc}
     *
     * @override
     * @see info.jmonit.visitor.MonitorVisitor#visit(info.jmonit.Monitor)
     */
    public void visit( Monitor monitor )
    {
        if ( state == DOMAIN )
        {
            // First monitor in the domain
            writer.println( "<table>" );
            even = false;
            writer.println( "<tr>" );
            writer.println( "<th>Monitor</th>" );
            writer.println( "<th>Hits</th>" );
            writer.println( "<th>Total</th>" );
            writer.println( "<th>Min</th>" );
            writer.println( "<th>Max</th>" );
            writer.println( "<th>Average</th>" );
            writer.println( "<th>Active</th>" );
            writer.println( "<th>Max active</th>" );
            writer.println( "<th>Average active</th>" );
            writer.println( "<th>First use</th>" );
            writer.println( "<th>Last use</th>" );
            writer.println( "</tr>" );
        }

        state = MONITOR;
        String domain = URLEncoder.encode( monitor.getDomain().getName() );
        String name = URLEncoder.encode( monitor.getName() );
        writer.println( "<tr class='" + ( even ? "even" : "odd" )
                + "' onclick=\"document.location='Show?domain=" + domain
                + "&name=" + name + "'\">" );
        writer.println( "<td class='monitor'>" );
        writer.println( monitor.getName() );
        writer.println( "</td>" );
        even = !even;
    }

    /**
     * {@inheritDoc}
     *
     * @override
     * @see info.jmonit.visitor.MonitorVisitor#visit(info.jmonit.visitor.MonitorSnapshot)
     */
    public void visit( MonitorSnapshot snapshot )
    {
        writer.print( "<td>" );
        writer.print( snapshot.getHits() );
        writer.println( "</td>" );

        writer.print( "<td>" );
        writer.print( snapshot.getTotal() );
        writer.println( "</td>" );

        writer.print( "<td>" );
        writer.print( snapshot.getMin() );
        writer.println( "</td>" );

        writer.print( "<td>" );
        writer.print( snapshot.getMax() );
        writer.println( "</td>" );

        writer.print( "<td>" );
        writer.print( numberFormat.format( snapshot.getAverage() ) );
        writer.println( "</td>" );

        writer.print( "<td>" );
        writer.print( snapshot.getActive() );
        writer.println( "</td>" );

        writer.print( "<td>" );
        writer.print( snapshot.getMaxActive() );
        writer.println( "</td>" );

        writer.print( "<td>" );
        writer.print( numberFormat.format( snapshot.getLoad() ) );
        writer.println( "</td>" );

        writer.print( "<td class='date'>" );
        writer.print( dateFormat.format( snapshot.getFirstUse() ) );
        writer.println( "</td>" );

        writer.print( "<td class='date'>" );
        writer.print( dateFormat.format( snapshot.getLastUse() ) );
        writer.println( "</td>" );

        writer.println( "</tr>" );
    }

    /**
     * {@inheritDoc}
     *
     * @see info.jmonit.visitor.MonitorVisitor#visit(info.jmonit.Agregator)
     */
    public void visit( Agregator agregator )
    {
        // Nop
    }

    /**
     * @param writer
     *            the writer to set
     */
    protected void setWriter( PrintWriter writer )
    {
        this.writer = writer;
    }

}
