/*
 ~ Copyright 2006-2007 Nicolas De Loof.
 ~
 ~ Licensed under the Apache License, Version 2.0 (the "License");
 ~ you may not use this file except in compliance with the License.
 ~ You may obtain a copy of the License at
 ~
 ~      http://www.apache.org/licenses/LICENSE-2.0
 ~
 ~ Unless required by applicable law or agreed to in writing, software
 ~ distributed under the License is distributed on an "AS IS" BASIS,
 ~ WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 ~ See the License for the specific language governing permissions and
 ~ limitations under the License.
 */
package info.jmonit.web.ui;

import info.jmonit.config.runtime.FactorySupport;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.Properties;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

/**
 * A simple controller implementation. Handles static resources (prefixed by
 * "/resources") and delegates to templates (build using Eclispe JET) for
 * rendering view.
 *
 * @author <a href="mailto:ndeloof@sourceforge.net">ndeloof</a>
 */
public class SimpleController
    implements Controller
{
    /**
     * {@inheritDoc}
     *
     * @see info.jmonit.web.ui.Controller#init(java.util.Properties)
     */
    public void init( Properties properties )
    {
        // nop
    }

    /**
     * {@inheritDoc}
     *
     * @see info.jmonit.web.ui.Controller#execute(java.lang.String,
     * javax.servlet.http.HttpServletRequest,
     * javax.servlet.http.HttpServletResponse)
     */
    public void execute( String uri, HttpServletRequest request, HttpServletResponse response )
        throws IOException, ServletException
    {
        // Plugable security
        if ( !checkSecurity( request, response ) )
        {
            response.sendError( HttpServletResponse.SC_UNAUTHORIZED );
            return;
        }

        if ( uri.startsWith( "resources/" ) )
        {
            getResource( uri, response );
            return;
        }
        
        try
        {
            WebComponent webComponent = getWebComponent( uri );
            webComponent.render( request, response );
            return;
        }
        catch ( Exception e )
        {
            response.sendError( HttpServletResponse.SC_NOT_FOUND );
        }
    }

    /**
     * Creates the template instnace for a requested URI. User can override this
     * method to provide an alternative templating engine.
     *
     * @param uri requested resources
     * @return WebComponent instance (can be null)
     */
    protected WebComponent getWebComponent( String uri )
    {
        return (WebComponent) FactorySupport.silentlyCreate( webComponentsPackage + "." + uri );
    }


    /**
     * Check for web ui security
     *
     * @param request incoming request
     * @param response Http response
     * @return tru if security check was OK
     * @throws IOException IO error
     * @throws ServletException internal error
     */
    protected boolean checkSecurity( HttpServletRequest request, HttpServletResponse response )
        throws IOException, ServletException
    {
        return true;
    }

    /**
     * Handle static resources, included in jMonit jar
     *
     * @param uri requested resource
     * @param response Http reponse
     * @throws IOException IO error
     */
    private void getResource( String uri, HttpServletResponse response )
        throws IOException
    {
        ClassLoader cl = Thread.currentThread().getContextClassLoader();
        if ( cl == null )
        {
            cl = SimpleController.class.getClassLoader();
        }
        InputStream input = cl.getResourceAsStream( "info/jmonit/web/ui/" + uri );
        if ( input != null )
        {
            OutputStream output = response.getOutputStream();
            copy( input, output );
        }
        else
        {
            response.sendError( HttpServletResponse.SC_NOT_FOUND );
        }
    }

    /** Buffer size for stream copy */
    private static final int BUFFER_SIZE = 1024 * 4;

    /** Templates package */
    private String webComponentsPackage = "info.jmonit.web.components";

    /**
     * Code from jakarta/commons-io 1.2
     *
     * @param input resource stream
     * @param output target stream
     * @throws IOException IO error
     */
    private void copy( InputStream input, OutputStream output )
        throws IOException
    {
        byte[] buffer = new byte[BUFFER_SIZE];
        int n = 0;
        while ( -1 != ( n = input.read( buffer ) ) )
        {
            output.write( buffer, 0, n );
        }
    }

    /**
     * @param webComponentsPackage the webComponentsPackage to set
     */
    public void setWebComponentsPackage( String webComponentsPackage )
    {
        this.webComponentsPackage = webComponentsPackage;
    }
}
